# $ANTLR 3.1.2 /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/cadlang.g 2012-09-12 14:02:23

import sys
from cadbiom.antlr3 import *
from cadbiom.antlr3.compat import set, frozenset

from cadbiom.antlr3.tree import *






# for convenience in actions
HIDDEN = BaseRecognizer.HIDDEN

# token types
ENDCONST=25
F=23
SYNC=26
RB=11
RP=18
LETTER=32
CONST=24
DEFAULT=15
LP=17
INPUT=9
T=22
ENDMACRO=14
EXCL=27
NOT=21
TEXT=12
AND=20
EOF=-1
SC=6
INC=28
NAME=4
WS=30
WHEN=16
COM=29
TARROW=7
OR=19
IDENT=5
LB=10
DIGIT=33
COMMENT=31
MACRO=13
PERM=8

# token names
tokenNames = [
    "<invalid>", "<EOR>", "<DOWN>", "<UP>",
    "NAME", "IDENT", "SC", "TARROW", "PERM", "INPUT", "LB", "RB", "TEXT",
    "MACRO", "ENDMACRO", "DEFAULT", "WHEN", "LP", "RP", "OR", "AND", "NOT",
    "T", "F", "CONST", "ENDCONST", "SYNC", "EXCL", "INC", "COM", "WS", "COMMENT",
    "LETTER", "DIGIT"
]




class cadlangParser(Parser):
    grammarFileName = "/home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/cadlang.g"
    antlr_version = version_str_to_tuple("3.1.2")
    antlr_version_str = "3.1.2"
    tokenNames = tokenNames

    def __init__(self, input, state=None):
        if state is None:
            state = RecognizerSharedState()

        Parser.__init__(self, input, state)








        self._adaptor = CommonTreeAdaptor()



    def getTreeAdaptor(self):
        return self._adaptor

    def setTreeAdaptor(self, adaptor):
        self._adaptor = adaptor

    adaptor = property(getTreeAdaptor, setTreeAdaptor)


    def set_error_reporter(self, err):
        self.error_reporter = err

    def displayRecognitionError(self, tokenNames, re):
        hdr = self.getErrorHeader(re)
        msg = self.getErrorMessage(re, tokenNames)
        self.error_reporter.display(hdr+' '+msg)


    def displayExceptionMessage(self, e):
        msg = self.getErrorMessage(self, e, tokenNames)
        self.error_reporter.display(msg)

    def check_ident_dec(self, id, type, line):
        """
        @param id : string
        """
        line_txt = 'line '+str(line)+':'
        try:
          place = self.symb_tab[id]
          self.error_reporter.display(line_txt+" Node double declaration:"+id)

        except KeyError:
          if type == 'S':
            node = self.current_macro.add_simple_node(id, 0,0)
          elif type == 'P':
            node = self.current_macro.add_perm_node(id, 0,0)
          elif type == 'I':
            node = self.current_macro.add_input_node(id, 0,0)
          else:
            self.error_reporter.display(line_txt+ " Unknown type - bug?")
          self.symb_tab[id] = node


    def check_ident_trans(self, id, type, line):
        """
        @param id : string
        """
        line_txt = 'line '+str(line)+':'
        try:
          # already used node
          node = self.symb_tab[id]
          tok = (node.is_input() and type == 'I') or (node.is_perm() and type == 'P') or (type == 'N')
          if (not tok)  :
            self.error_reporter.display(line_txt + " Type error in transition: "+id)
        except KeyError:
          if type == 'N':
            node = self.current_macro.add_simple_node(id, 0,0)
          elif type == 'P':
            node = self.current_macro.add_perm_node(id, 0,0)
          elif type == 'I':
            node = self.current_macro.add_input_node(id, 0,0)
          else:
            self.error_reporter.display(line_txt+ " Unknown type - bug?")
          self.symb_tab[id] = node
        return node

    def check_ident_start(self, id, line):
        """
        @param id : string
        """
        line_txt = 'line '+str(line)+':'
        try:
          # already used node, must be simple or macro
          node = self.symb_tab[id]
          tnok = node.is_input() or node.is_perm()
          if tnok:
            self.error_reporter.display(line_txt+ " Type error in start transition: "+id)
        except KeyError:
          node = self.current_macro.add_simple_node(id, 0,0)
          self.symb_tab[id] = node
        return node

    def check_ident_deg(self, id, line):
        """
        @param id : string
        """
        line_txt = 'line '+str(line)+':'
        try:
          # already used node, must be simple
          node = self.symb_tab[id]
          if not (node.is_simple() or node.is_macro()):
            # degradation allowed from macro nodes - TO MODIFY??
            self.error_reporter.display(line_txt+" Type error in degradation transition: "+id)
        except KeyError:
          node = self.current_macro.add_simple_node(id, 0,0)
          self.symb_tab[id] = node
        return node

    def build_transition(self, id1, modif, id2, modif2, gc, note, line):
        line_txt = 'line '+str(line)+':'
        if modif:
          type = self.modif_code(modif)
        else:
          type = 'N'
        ori = self.check_ident_trans(id1, type, line)
        if modif2:
          type2 = self.modif_code(modif)
        else:
          type2 = 'N'
        target = self.check_ident_trans(id2, type2, line)
        # check if origin and target are in the same macro
        if not ori.father == target.father:
          self.error_reporter.display(line_txt+" Node:"+id1+ " and node:"+id2 + " are not in the same macro")
        if gc[0]:
          event = gc[0]
        else:
          event = ''
        if gc[1]:
          cond = gc[1]
        else:
          cond = ''
        note_txt = self.clean_note(note)
        t = self.current_macro.add_transition(ori, target)
        if event:
          t.set_event(event)
        t.set_condition(cond)
        if len(note_txt)>0:
          t.note = note_txt

    def build_start_transition(self, id, note, line):
        ori = self.current_macro.add_start_node(0,0)
        target = self.check_ident_start(id, line)
        t = self.current_macro.add_transition(ori, target)
        if note:
          note_txt = self.clean_note(note)
          t.set_note(note_txt)

    def build_deg_transition(self, id, gc, note, line):
        target = self.current_macro.add_trap_node(0,0)
        ori = self.check_ident_deg(id, line)
        event = gc[0]
        cond = gc[1]
        note_txt = self.clean_note(note)
        t = self.current_macro.add_transition(ori, target)
        if event:
          t.set_event(event)
        t.set_condition(cond)

    def enter_macro(self, id, line):
        line_txt = 'line '+str(line)+':'
        self.macro_pile.append(self.current_macro)
        try:
          node = self.symb_tab[id]
          if not node.is_macro():
            self.error_reporter.display(line_txt+ " Not macro node used as macro:"+id)
            return
          else:
            self.error_reporter.display(line_txt+ " Macro double definition:"+id)

        except KeyError:
          node = self.current_macro.add_macro_subnode(id,0,0, 0.25, 0.25)
          self.symb_tab[id] = node
        self.current_macro = node


    def leave_macro(self, id=None):
        self.current_macro = self.macro_pile.pop()

    def clean_note(self, note):
        return note[1:-1]

    def modif_code(self, modif):
        if modif == '/p':
          return 'P'
        elif modif == '/i':
          return 'I'

    def check_end(self):
        if len(self.macro_pile)>0:
          self.error_reporter.display("Bad macro imbrication - missing \endmacro?")


    class cad_model_return(ParserRuleReturnScope):
        def __init__(self):
            ParserRuleReturnScope.__init__(self)

            self.tree = None




    # $ANTLR start "cad_model"
    # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/cadlang.g:207:1: cad_model[model] : ( NAME id= IDENT )? ( macro | transition | dec )+ (txt= constraints )? EOF ;
    def cad_model(self, model):

        retval = self.cad_model_return()
        retval.start = self.input.LT(1)

        root_0 = None

        id = None
        NAME1 = None
        EOF5 = None
        txt = None

        macro2 = None

        transition3 = None

        dec4 = None


        id_tree = None
        NAME1_tree = None
        EOF5_tree = None


        self.model = model
        self.macro_pile = []
        self.symb_tab = dict()
        try:
            try:
                # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/cadlang.g:212:12: ( ( NAME id= IDENT )? ( macro | transition | dec )+ (txt= constraints )? EOF )
                # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/cadlang.g:213:17: ( NAME id= IDENT )? ( macro | transition | dec )+ (txt= constraints )? EOF
                pass
                root_0 = self._adaptor.nil()

                #action start
                self.current_macro = self.model.get_root()
                #action end
                # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/cadlang.g:215:13: ( NAME id= IDENT )?
                alt1 = 2
                LA1_0 = self.input.LA(1)

                if (LA1_0 == NAME) :
                    alt1 = 1
                if alt1 == 1:
                    # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/cadlang.g:215:14: NAME id= IDENT
                    pass
                    NAME1=self.match(self.input, NAME, self.FOLLOW_NAME_in_cad_model139)

                    NAME1_tree = self._adaptor.createWithPayload(NAME1)
                    self._adaptor.addChild(root_0, NAME1_tree)

                    id=self.match(self.input, IDENT, self.FOLLOW_IDENT_in_cad_model143)

                    id_tree = self._adaptor.createWithPayload(id)
                    self._adaptor.addChild(root_0, id_tree)

                    #action start
                    self.model.name = id.text
                    #action end



                # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/cadlang.g:216:13: ( macro | transition | dec )+
                cnt2 = 0
                while True: #loop2
                    alt2 = 4
                    LA2 = self.input.LA(1)
                    if LA2 == MACRO:
                        alt2 = 1
                    elif LA2 == IDENT:
                        LA2 = self.input.LA(2)
                        if LA2 == TARROW:
                            alt2 = 2
                        elif LA2 == PERM or LA2 == INPUT:
                            LA2_5 = self.input.LA(3)

                            if (LA2_5 == SC) :
                                alt2 = 3
                            elif (LA2_5 == TARROW) :
                                alt2 = 2


                        elif LA2 == SC:
                            alt2 = 3

                    elif LA2 == TARROW:
                        alt2 = 2

                    if alt2 == 1:
                        # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/cadlang.g:216:14: macro
                        pass
                        self._state.following.append(self.FOLLOW_macro_in_cad_model162)
                        macro2 = self.macro()

                        self._state.following.pop()
                        self._adaptor.addChild(root_0, macro2.tree)


                    elif alt2 == 2:
                        # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/cadlang.g:216:20: transition
                        pass
                        self._state.following.append(self.FOLLOW_transition_in_cad_model164)
                        transition3 = self.transition()

                        self._state.following.pop()
                        self._adaptor.addChild(root_0, transition3.tree)


                    elif alt2 == 3:
                        # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/cadlang.g:216:31: dec
                        pass
                        self._state.following.append(self.FOLLOW_dec_in_cad_model166)
                        dec4 = self.dec()

                        self._state.following.pop()
                        self._adaptor.addChild(root_0, dec4.tree)


                    else:
                        if cnt2 >= 1:
                            break #loop2

                        eee = EarlyExitException(2, self.input)
                        raise eee

                    cnt2 += 1


                #action start
                self.check_end()
                #action end
                # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/cadlang.g:218:13: (txt= constraints )?
                alt3 = 2
                LA3_0 = self.input.LA(1)

                if (LA3_0 == CONST) :
                    alt3 = 1
                if alt3 == 1:
                    # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/cadlang.g:218:14: txt= constraints
                    pass
                    self._state.following.append(self.FOLLOW_constraints_in_cad_model203)
                    txt = self.constraints()

                    self._state.following.pop()
                    self._adaptor.addChild(root_0, txt.tree)
                    #action start
                    self.model.constraints = ((txt is not None) and [txt.text] or [None])[0]
                    #action end



                EOF5=self.match(self.input, EOF, self.FOLLOW_EOF_in_cad_model222)

                EOF5_tree = self._adaptor.createWithPayload(EOF5)
                self._adaptor.addChild(root_0, EOF5_tree)




                retval.stop = self.input.LT(-1)


                retval.tree = self._adaptor.rulePostProcessing(root_0)
                self._adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop)


            except RecognitionException, re:
                self.reportError(re)
                self.recover(self.input, re)
                retval.tree = self._adaptor.errorNode(self.input, retval.start, self.input.LT(-1), re)
        finally:

            pass

        return retval

    # $ANTLR end "cad_model"

    class dec_return(ParserRuleReturnScope):
        def __init__(self):
            ParserRuleReturnScope.__init__(self)

            self.tree = None




    # $ANTLR start "dec"
    # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/cadlang.g:222:1: dec : id= IDENT (m= modifier )? SC ;
    def dec(self, ):

        retval = self.dec_return()
        retval.start = self.input.LT(1)

        root_0 = None

        id = None
        SC6 = None
        m = None


        id_tree = None
        SC6_tree = None

        try:
            try:
                # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/cadlang.g:222:12: (id= IDENT (m= modifier )? SC )
                # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/cadlang.g:222:13: id= IDENT (m= modifier )? SC
                pass
                root_0 = self._adaptor.nil()

                #action start
                type = 'S'
                #action end
                id=self.match(self.input, IDENT, self.FOLLOW_IDENT_in_dec270)

                id_tree = self._adaptor.createWithPayload(id)
                self._adaptor.addChild(root_0, id_tree)

                # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/cadlang.g:224:22: (m= modifier )?
                alt4 = 2
                LA4_0 = self.input.LA(1)

                if ((PERM <= LA4_0 <= INPUT)) :
                    alt4 = 1
                if alt4 == 1:
                    # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/cadlang.g:224:23: m= modifier
                    pass
                    self._state.following.append(self.FOLLOW_modifier_in_dec275)
                    m = self.modifier()

                    self._state.following.pop()
                    self._adaptor.addChild(root_0, m.tree)
                    #action start
                    type = self.modif_code(((m is not None) and [self.input.toString(m.start,m.stop)] or [None])[0])
                    #action end



                SC6=self.match(self.input, SC, self.FOLLOW_SC_in_dec281)

                SC6_tree = self._adaptor.createWithPayload(SC6)
                self._adaptor.addChild(root_0, SC6_tree)

                #action start
                self.check_ident_dec(id.text, type, id.line)
                #action end



                retval.stop = self.input.LT(-1)


                retval.tree = self._adaptor.rulePostProcessing(root_0)
                self._adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop)


            except RecognitionException, re:
                self.reportError(re)
                self.recover(self.input, re)
                retval.tree = self._adaptor.errorNode(self.input, retval.start, self.input.LT(-1), re)
        finally:

            pass

        return retval

    # $ANTLR end "dec"

    class transition_return(ParserRuleReturnScope):
        def __init__(self):
            ParserRuleReturnScope.__init__(self)

            self.tree = None




    # $ANTLR start "transition"
    # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/cadlang.g:228:1: transition : (id1= IDENT (m= modifier )? TARROW id2= IDENT SC gc= guard (m2= modifier )? ntext= note SC | id3= IDENT TARROW SC gc1= guard ntext1= note SC | TARROW id4= IDENT SC (ntext2= note SC )? );
    def transition(self, ):

        retval = self.transition_return()
        retval.start = self.input.LT(1)

        root_0 = None

        id1 = None
        id2 = None
        id3 = None
        id4 = None
        TARROW7 = None
        SC8 = None
        SC9 = None
        TARROW10 = None
        SC11 = None
        SC12 = None
        TARROW13 = None
        SC14 = None
        SC15 = None
        m = None

        gc = None

        m2 = None

        ntext = None

        gc1 = None

        ntext1 = None

        ntext2 = None


        id1_tree = None
        id2_tree = None
        id3_tree = None
        id4_tree = None
        TARROW7_tree = None
        SC8_tree = None
        SC9_tree = None
        TARROW10_tree = None
        SC11_tree = None
        SC12_tree = None
        TARROW13_tree = None
        SC14_tree = None
        SC15_tree = None

        try:
            try:
                # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/cadlang.g:228:12: (id1= IDENT (m= modifier )? TARROW id2= IDENT SC gc= guard (m2= modifier )? ntext= note SC | id3= IDENT TARROW SC gc1= guard ntext1= note SC | TARROW id4= IDENT SC (ntext2= note SC )? )
                alt8 = 3
                LA8_0 = self.input.LA(1)

                if (LA8_0 == IDENT) :
                    LA8_1 = self.input.LA(2)

                    if (LA8_1 == TARROW) :
                        LA8_3 = self.input.LA(3)

                        if (LA8_3 == SC) :
                            alt8 = 2
                        elif (LA8_3 == IDENT) :
                            alt8 = 1
                        else:
                            nvae = NoViableAltException("", 8, 3, self.input)

                            raise nvae

                    elif ((PERM <= LA8_1 <= INPUT)) :
                        alt8 = 1
                    else:
                        nvae = NoViableAltException("", 8, 1, self.input)

                        raise nvae

                elif (LA8_0 == TARROW) :
                    alt8 = 3
                else:
                    nvae = NoViableAltException("", 8, 0, self.input)

                    raise nvae

                if alt8 == 1:
                    # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/cadlang.g:229:13: id1= IDENT (m= modifier )? TARROW id2= IDENT SC gc= guard (m2= modifier )? ntext= note SC
                    pass
                    root_0 = self._adaptor.nil()

                    id1=self.match(self.input, IDENT, self.FOLLOW_IDENT_in_transition340)

                    id1_tree = self._adaptor.createWithPayload(id1)
                    self._adaptor.addChild(root_0, id1_tree)

                    # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/cadlang.g:229:23: (m= modifier )?
                    alt5 = 2
                    LA5_0 = self.input.LA(1)

                    if ((PERM <= LA5_0 <= INPUT)) :
                        alt5 = 1
                    if alt5 == 1:
                        # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/cadlang.g:229:24: m= modifier
                        pass
                        self._state.following.append(self.FOLLOW_modifier_in_transition345)
                        m = self.modifier()

                        self._state.following.pop()
                        self._adaptor.addChild(root_0, m.tree)



                    TARROW7=self.match(self.input, TARROW, self.FOLLOW_TARROW_in_transition349)

                    TARROW7_tree = self._adaptor.createWithPayload(TARROW7)
                    self._adaptor.addChild(root_0, TARROW7_tree)

                    id2=self.match(self.input, IDENT, self.FOLLOW_IDENT_in_transition353)

                    id2_tree = self._adaptor.createWithPayload(id2)
                    self._adaptor.addChild(root_0, id2_tree)

                    SC8=self.match(self.input, SC, self.FOLLOW_SC_in_transition355)

                    SC8_tree = self._adaptor.createWithPayload(SC8)
                    self._adaptor.addChild(root_0, SC8_tree)

                    self._state.following.append(self.FOLLOW_guard_in_transition359)
                    gc = self.guard()

                    self._state.following.pop()
                    self._adaptor.addChild(root_0, gc.tree)
                    # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/cadlang.g:229:66: (m2= modifier )?
                    alt6 = 2
                    LA6_0 = self.input.LA(1)

                    if ((PERM <= LA6_0 <= INPUT)) :
                        alt6 = 1
                    if alt6 == 1:
                        # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/cadlang.g:229:67: m2= modifier
                        pass
                        self._state.following.append(self.FOLLOW_modifier_in_transition364)
                        m2 = self.modifier()

                        self._state.following.pop()
                        self._adaptor.addChild(root_0, m2.tree)



                    self._state.following.append(self.FOLLOW_note_in_transition370)
                    ntext = self.note()

                    self._state.following.pop()
                    self._adaptor.addChild(root_0, ntext.tree)
                    SC9=self.match(self.input, SC, self.FOLLOW_SC_in_transition372)

                    SC9_tree = self._adaptor.createWithPayload(SC9)
                    self._adaptor.addChild(root_0, SC9_tree)

                    #action start
                    self.build_transition(id1.text, ((m is not None) and [self.input.toString(m.start,m.stop)] or [None])[0], id2.text, ((m2 is not None) and [self.input.toString(m2.start,m2.stop)] or [None])[0], ((gc is not None) and [gc.guard_component] or [None])[0], ((ntext is not None) and [self.input.toString(ntext.start,ntext.stop)] or [None])[0], id1.line)
                    #action end


                elif alt8 == 2:
                    # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/cadlang.g:232:14: id3= IDENT TARROW SC gc1= guard ntext1= note SC
                    pass
                    root_0 = self._adaptor.nil()

                    id3=self.match(self.input, IDENT, self.FOLLOW_IDENT_in_transition416)

                    id3_tree = self._adaptor.createWithPayload(id3)
                    self._adaptor.addChild(root_0, id3_tree)

                    TARROW10=self.match(self.input, TARROW, self.FOLLOW_TARROW_in_transition418)

                    TARROW10_tree = self._adaptor.createWithPayload(TARROW10)
                    self._adaptor.addChild(root_0, TARROW10_tree)

                    SC11=self.match(self.input, SC, self.FOLLOW_SC_in_transition421)

                    SC11_tree = self._adaptor.createWithPayload(SC11)
                    self._adaptor.addChild(root_0, SC11_tree)

                    self._state.following.append(self.FOLLOW_guard_in_transition425)
                    gc1 = self.guard()

                    self._state.following.pop()
                    self._adaptor.addChild(root_0, gc1.tree)
                    self._state.following.append(self.FOLLOW_note_in_transition429)
                    ntext1 = self.note()

                    self._state.following.pop()
                    self._adaptor.addChild(root_0, ntext1.tree)
                    SC12=self.match(self.input, SC, self.FOLLOW_SC_in_transition431)

                    SC12_tree = self._adaptor.createWithPayload(SC12)
                    self._adaptor.addChild(root_0, SC12_tree)

                    #action start
                    self.build_deg_transition(id3.text, ((gc1 is not None) and [gc1.guard_component] or [None])[0], ((ntext1 is not None) and [self.input.toString(ntext1.start,ntext1.stop)] or [None])[0], id3.line)
                    #action end


                elif alt8 == 3:
                    # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/cadlang.g:235:14: TARROW id4= IDENT SC (ntext2= note SC )?
                    pass
                    root_0 = self._adaptor.nil()

                    TARROW13=self.match(self.input, TARROW, self.FOLLOW_TARROW_in_transition474)

                    TARROW13_tree = self._adaptor.createWithPayload(TARROW13)
                    self._adaptor.addChild(root_0, TARROW13_tree)

                    id4=self.match(self.input, IDENT, self.FOLLOW_IDENT_in_transition478)

                    id4_tree = self._adaptor.createWithPayload(id4)
                    self._adaptor.addChild(root_0, id4_tree)

                    SC14=self.match(self.input, SC, self.FOLLOW_SC_in_transition481)

                    SC14_tree = self._adaptor.createWithPayload(SC14)
                    self._adaptor.addChild(root_0, SC14_tree)

                    # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/cadlang.g:235:35: (ntext2= note SC )?
                    alt7 = 2
                    LA7_0 = self.input.LA(1)

                    if (LA7_0 == SC or LA7_0 == TEXT) :
                        alt7 = 1
                    if alt7 == 1:
                        # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/cadlang.g:235:36: ntext2= note SC
                        pass
                        self._state.following.append(self.FOLLOW_note_in_transition486)
                        ntext2 = self.note()

                        self._state.following.pop()
                        self._adaptor.addChild(root_0, ntext2.tree)
                        SC15=self.match(self.input, SC, self.FOLLOW_SC_in_transition488)

                        SC15_tree = self._adaptor.createWithPayload(SC15)
                        self._adaptor.addChild(root_0, SC15_tree)




                    #action start
                    self.build_start_transition(id4.text, ((ntext2 is not None) and [self.input.toString(ntext2.start,ntext2.stop)] or [None])[0], id4.line)
                    #action end


                retval.stop = self.input.LT(-1)


                retval.tree = self._adaptor.rulePostProcessing(root_0)
                self._adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop)


            except RecognitionException, re:
                self.reportError(re)
                self.recover(self.input, re)
                retval.tree = self._adaptor.errorNode(self.input, retval.start, self.input.LT(-1), re)
        finally:

            pass

        return retval

    # $ANTLR end "transition"

    class modifier_return(ParserRuleReturnScope):
        def __init__(self):
            ParserRuleReturnScope.__init__(self)

            self.tree = None




    # $ANTLR start "modifier"
    # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/cadlang.g:239:1: modifier : ( PERM | INPUT );
    def modifier(self, ):

        retval = self.modifier_return()
        retval.start = self.input.LT(1)

        root_0 = None

        set16 = None

        set16_tree = None

        try:
            try:
                # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/cadlang.g:239:12: ( PERM | INPUT )
                # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/cadlang.g:
                pass
                root_0 = self._adaptor.nil()

                set16 = self.input.LT(1)
                if (PERM <= self.input.LA(1) <= INPUT):
                    self.input.consume()
                    self._adaptor.addChild(root_0, self._adaptor.createWithPayload(set16))
                    self._state.errorRecovery = False

                else:
                    mse = MismatchedSetException(None, self.input)
                    raise mse





                retval.stop = self.input.LT(-1)


                retval.tree = self._adaptor.rulePostProcessing(root_0)
                self._adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop)


            except RecognitionException, re:
                self.reportError(re)
                self.recover(self.input, re)
                retval.tree = self._adaptor.errorNode(self.input, retval.start, self.input.LT(-1), re)
        finally:

            pass

        return retval

    # $ANTLR end "modifier"

    class guard_return(ParserRuleReturnScope):
        def __init__(self):
            ParserRuleReturnScope.__init__(self)

            self.guard_component = None
            self.tree = None




    # $ANTLR start "guard"
    # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/cadlang.g:245:1: guard returns [guard_component] : (h= sig_expression )? LB (bt= bool_exp ) RB (id2= IDENT )? ;
    def guard(self, ):

        retval = self.guard_return()
        retval.start = self.input.LT(1)

        root_0 = None

        id2 = None
        LB17 = None
        RB18 = None
        h = None

        bt = None


        id2_tree = None
        LB17_tree = None
        RB18_tree = None

        try:
            try:
                # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/cadlang.g:246:12: ( (h= sig_expression )? LB (bt= bool_exp ) RB (id2= IDENT )? )
                # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/cadlang.g:247:13: (h= sig_expression )? LB (bt= bool_exp ) RB (id2= IDENT )?
                pass
                root_0 = self._adaptor.nil()

                # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/cadlang.g:247:13: (h= sig_expression )?
                alt9 = 2
                LA9_0 = self.input.LA(1)

                if (LA9_0 == IDENT or LA9_0 == LP) :
                    alt9 = 1
                if alt9 == 1:
                    # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/cadlang.g:247:14: h= sig_expression
                    pass
                    self._state.following.append(self.FOLLOW_sig_expression_in_guard634)
                    h = self.sig_expression()

                    self._state.following.pop()
                    self._adaptor.addChild(root_0, h.tree)



                LB17=self.match(self.input, LB, self.FOLLOW_LB_in_guard638)

                LB17_tree = self._adaptor.createWithPayload(LB17)
                self._adaptor.addChild(root_0, LB17_tree)

                # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/cadlang.g:247:36: (bt= bool_exp )
                # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/cadlang.g:247:37: bt= bool_exp
                pass
                self._state.following.append(self.FOLLOW_bool_exp_in_guard643)
                bt = self.bool_exp()

                self._state.following.pop()
                self._adaptor.addChild(root_0, bt.tree)



                RB18=self.match(self.input, RB, self.FOLLOW_RB_in_guard646)

                RB18_tree = self._adaptor.createWithPayload(RB18)
                self._adaptor.addChild(root_0, RB18_tree)

                # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/cadlang.g:247:53: (id2= IDENT )?
                alt10 = 2
                LA10_0 = self.input.LA(1)

                if (LA10_0 == IDENT) :
                    alt10 = 1
                if alt10 == 1:
                    # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/cadlang.g:247:54: id2= IDENT
                    pass
                    id2=self.match(self.input, IDENT, self.FOLLOW_IDENT_in_guard651)

                    id2_tree = self._adaptor.createWithPayload(id2)
                    self._adaptor.addChild(root_0, id2_tree)




                #action start
                if h and id2: retval.guard_component = (((h is not None) and [h.text] or [None])[0], ((bt is not None) and [bt.text] or [None])[0], id2.text)
                elif h: retval.guard_component = (((h is not None) and [h.text] or [None])[0], ((bt is not None) and [bt.text] or [None])[0], None)
                elif id2: retval.guard_component = (None, ((bt is not None) and [bt.text] or [None])[0], id2.text)
                else:  retval.guard_component = (None, ((bt is not None) and [bt.text] or [None])[0], None)
                #action end



                retval.stop = self.input.LT(-1)


                retval.tree = self._adaptor.rulePostProcessing(root_0)
                self._adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop)


            except RecognitionException, re:
                self.reportError(re)
                self.recover(self.input, re)
                retval.tree = self._adaptor.errorNode(self.input, retval.start, self.input.LT(-1), re)
        finally:

            pass

        return retval

    # $ANTLR end "guard"

    class note_return(ParserRuleReturnScope):
        def __init__(self):
            ParserRuleReturnScope.__init__(self)

            self.tree = None




    # $ANTLR start "note"
    # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/cadlang.g:254:1: note : ( TEXT | );
    def note(self, ):

        retval = self.note_return()
        retval.start = self.input.LT(1)

        root_0 = None

        TEXT19 = None

        TEXT19_tree = None

        try:
            try:
                # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/cadlang.g:254:12: ( TEXT | )
                alt11 = 2
                LA11_0 = self.input.LA(1)

                if (LA11_0 == TEXT) :
                    alt11 = 1
                elif (LA11_0 == SC) :
                    alt11 = 2
                else:
                    nvae = NoViableAltException("", 11, 0, self.input)

                    raise nvae

                if alt11 == 1:
                    # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/cadlang.g:255:13: TEXT
                    pass
                    root_0 = self._adaptor.nil()

                    TEXT19=self.match(self.input, TEXT, self.FOLLOW_TEXT_in_note717)

                    TEXT19_tree = self._adaptor.createWithPayload(TEXT19)
                    self._adaptor.addChild(root_0, TEXT19_tree)



                elif alt11 == 2:
                    # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/cadlang.g:257:12:
                    pass
                    root_0 = self._adaptor.nil()


                retval.stop = self.input.LT(-1)


                retval.tree = self._adaptor.rulePostProcessing(root_0)
                self._adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop)


            except RecognitionException, re:
                self.reportError(re)
                self.recover(self.input, re)
                retval.tree = self._adaptor.errorNode(self.input, retval.start, self.input.LT(-1), re)
        finally:

            pass

        return retval

    # $ANTLR end "note"

    class macro_return(ParserRuleReturnScope):
        def __init__(self):
            ParserRuleReturnScope.__init__(self)

            self.tree = None




    # $ANTLR start "macro"
    # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/cadlang.g:259:1: macro : MACRO id1= IDENT ( macro | transition | dec )* ENDMACRO ;
    def macro(self, ):

        retval = self.macro_return()
        retval.start = self.input.LT(1)

        root_0 = None

        id1 = None
        MACRO20 = None
        ENDMACRO24 = None
        macro21 = None

        transition22 = None

        dec23 = None


        id1_tree = None
        MACRO20_tree = None
        ENDMACRO24_tree = None

        id2 = None
        try:
            try:
                # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/cadlang.g:260:12: ( MACRO id1= IDENT ( macro | transition | dec )* ENDMACRO )
                # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/cadlang.g:261:13: MACRO id1= IDENT ( macro | transition | dec )* ENDMACRO
                pass
                root_0 = self._adaptor.nil()

                MACRO20=self.match(self.input, MACRO, self.FOLLOW_MACRO_in_macro795)

                MACRO20_tree = self._adaptor.createWithPayload(MACRO20)
                self._adaptor.addChild(root_0, MACRO20_tree)

                id1=self.match(self.input, IDENT, self.FOLLOW_IDENT_in_macro799)

                id1_tree = self._adaptor.createWithPayload(id1)
                self._adaptor.addChild(root_0, id1_tree)

                #action start
                self.enter_macro(id1.text, id1.line)
                #action end
                # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/cadlang.g:263:13: ( macro | transition | dec )*
                while True: #loop12
                    alt12 = 4
                    LA12 = self.input.LA(1)
                    if LA12 == MACRO:
                        alt12 = 1
                    elif LA12 == IDENT:
                        LA12 = self.input.LA(2)
                        if LA12 == TARROW:
                            alt12 = 2
                        elif LA12 == PERM or LA12 == INPUT:
                            LA12_5 = self.input.LA(3)

                            if (LA12_5 == TARROW) :
                                alt12 = 2
                            elif (LA12_5 == SC) :
                                alt12 = 3


                        elif LA12 == SC:
                            alt12 = 3

                    elif LA12 == TARROW:
                        alt12 = 2

                    if alt12 == 1:
                        # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/cadlang.g:263:14: macro
                        pass
                        self._state.following.append(self.FOLLOW_macro_in_macro829)
                        macro21 = self.macro()

                        self._state.following.pop()
                        self._adaptor.addChild(root_0, macro21.tree)


                    elif alt12 == 2:
                        # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/cadlang.g:263:20: transition
                        pass
                        self._state.following.append(self.FOLLOW_transition_in_macro831)
                        transition22 = self.transition()

                        self._state.following.pop()
                        self._adaptor.addChild(root_0, transition22.tree)


                    elif alt12 == 3:
                        # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/cadlang.g:263:31: dec
                        pass
                        self._state.following.append(self.FOLLOW_dec_in_macro833)
                        dec23 = self.dec()

                        self._state.following.pop()
                        self._adaptor.addChild(root_0, dec23.tree)


                    else:
                        break #loop12


                ENDMACRO24=self.match(self.input, ENDMACRO, self.FOLLOW_ENDMACRO_in_macro849)

                ENDMACRO24_tree = self._adaptor.createWithPayload(ENDMACRO24)
                self._adaptor.addChild(root_0, ENDMACRO24_tree)

                #action start
                self.leave_macro()
                #action end



                retval.stop = self.input.LT(-1)


                retval.tree = self._adaptor.rulePostProcessing(root_0)
                self._adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop)


            except RecognitionException, re:
                self.reportError(re)
                self.recover(self.input, re)
                retval.tree = self._adaptor.errorNode(self.input, retval.start, self.input.LT(-1), re)
        finally:

            pass

        return retval

    # $ANTLR end "macro"

    class sig_expression_return(ParserRuleReturnScope):
        def __init__(self):
            ParserRuleReturnScope.__init__(self)

            self.text = None
            self.tree = None




    # $ANTLR start "sig_expression"
    # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/cadlang.g:269:1: sig_expression returns [text] : exp1= sig_exp ( DEFAULT exp2= sig_exp )* ;
    def sig_expression(self, ):

        retval = self.sig_expression_return()
        retval.start = self.input.LT(1)

        root_0 = None

        DEFAULT25 = None
        exp1 = None

        exp2 = None


        DEFAULT25_tree = None

        try:
            try:
                # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/cadlang.g:270:9: (exp1= sig_exp ( DEFAULT exp2= sig_exp )* )
                # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/cadlang.g:270:11: exp1= sig_exp ( DEFAULT exp2= sig_exp )*
                pass
                root_0 = self._adaptor.nil()

                self._state.following.append(self.FOLLOW_sig_exp_in_sig_expression923)
                exp1 = self.sig_exp()

                self._state.following.pop()
                self._adaptor.addChild(root_0, exp1.tree)
                #action start
                retval.text = ((exp1 is not None) and [exp1.text] or [None])[0]
                #action end
                # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/cadlang.g:271:11: ( DEFAULT exp2= sig_exp )*
                while True: #loop13
                    alt13 = 2
                    LA13_0 = self.input.LA(1)

                    if (LA13_0 == DEFAULT) :
                        alt13 = 1


                    if alt13 == 1:
                        # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/cadlang.g:271:12: DEFAULT exp2= sig_exp
                        pass
                        DEFAULT25=self.match(self.input, DEFAULT, self.FOLLOW_DEFAULT_in_sig_expression939)

                        DEFAULT25_tree = self._adaptor.createWithPayload(DEFAULT25)
                        root_0 = self._adaptor.becomeRoot(DEFAULT25_tree, root_0)

                        self._state.following.append(self.FOLLOW_sig_exp_in_sig_expression944)
                        exp2 = self.sig_exp()

                        self._state.following.pop()
                        self._adaptor.addChild(root_0, exp2.tree)
                        #action start
                        retval.text = retval.text + ' default '+ ((exp2 is not None) and [exp2.text] or [None])[0]
                        #action end


                    else:
                        break #loop13





                retval.stop = self.input.LT(-1)


                retval.tree = self._adaptor.rulePostProcessing(root_0)
                self._adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop)


            except RecognitionException, re:
                self.reportError(re)
                self.recover(self.input, re)
                retval.tree = self._adaptor.errorNode(self.input, retval.start, self.input.LT(-1), re)
        finally:

            pass

        return retval

    # $ANTLR end "sig_expression"

    class sig_exp_return(ParserRuleReturnScope):
        def __init__(self):
            ParserRuleReturnScope.__init__(self)

            self.text = None
            self.tree = None




    # $ANTLR start "sig_exp"
    # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/cadlang.g:276:1: sig_exp returns [text] : exp1= sig_exp_primary ( WHEN exp2= bool_exp )* ;
    def sig_exp(self, ):

        retval = self.sig_exp_return()
        retval.start = self.input.LT(1)

        root_0 = None

        WHEN26 = None
        exp1 = None

        exp2 = None


        WHEN26_tree = None

        try:
            try:
                # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/cadlang.g:277:9: (exp1= sig_exp_primary ( WHEN exp2= bool_exp )* )
                # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/cadlang.g:277:12: exp1= sig_exp_primary ( WHEN exp2= bool_exp )*
                pass
                root_0 = self._adaptor.nil()

                self._state.following.append(self.FOLLOW_sig_exp_primary_in_sig_exp1014)
                exp1 = self.sig_exp_primary()

                self._state.following.pop()
                self._adaptor.addChild(root_0, exp1.tree)
                #action start
                retval.text = ((exp1 is not None) and [exp1.text] or [None])[0]
                #action end
                # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/cadlang.g:278:11: ( WHEN exp2= bool_exp )*
                while True: #loop14
                    alt14 = 2
                    LA14_0 = self.input.LA(1)

                    if (LA14_0 == WHEN) :
                        alt14 = 1


                    if alt14 == 1:
                        # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/cadlang.g:278:12: WHEN exp2= bool_exp
                        pass
                        WHEN26=self.match(self.input, WHEN, self.FOLLOW_WHEN_in_sig_exp1029)

                        WHEN26_tree = self._adaptor.createWithPayload(WHEN26)
                        root_0 = self._adaptor.becomeRoot(WHEN26_tree, root_0)

                        self._state.following.append(self.FOLLOW_bool_exp_in_sig_exp1034)
                        exp2 = self.bool_exp()

                        self._state.following.pop()
                        self._adaptor.addChild(root_0, exp2.tree)
                        #action start
                        retval.text = retval.text + ' when '+ ((exp2 is not None) and [exp2.text] or [None])[0]
                        #action end


                    else:
                        break #loop14





                retval.stop = self.input.LT(-1)


                retval.tree = self._adaptor.rulePostProcessing(root_0)
                self._adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop)


            except RecognitionException, re:
                self.reportError(re)
                self.recover(self.input, re)
                retval.tree = self._adaptor.errorNode(self.input, retval.start, self.input.LT(-1), re)
        finally:

            pass

        return retval

    # $ANTLR end "sig_exp"

    class sig_exp_primary_return(ParserRuleReturnScope):
        def __init__(self):
            ParserRuleReturnScope.__init__(self)

            self.text = None
            self.tree = None




    # $ANTLR start "sig_exp_primary"
    # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/cadlang.g:283:1: sig_exp_primary returns [text] : (id= IDENT | LP se= sig_expression RP );
    def sig_exp_primary(self, ):

        retval = self.sig_exp_primary_return()
        retval.start = self.input.LT(1)

        root_0 = None

        id = None
        LP27 = None
        RP28 = None
        se = None


        id_tree = None
        LP27_tree = None
        RP28_tree = None

        try:
            try:
                # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/cadlang.g:284:9: (id= IDENT | LP se= sig_expression RP )
                alt15 = 2
                LA15_0 = self.input.LA(1)

                if (LA15_0 == IDENT) :
                    alt15 = 1
                elif (LA15_0 == LP) :
                    alt15 = 2
                else:
                    nvae = NoViableAltException("", 15, 0, self.input)

                    raise nvae

                if alt15 == 1:
                    # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/cadlang.g:284:11: id= IDENT
                    pass
                    root_0 = self._adaptor.nil()

                    id=self.match(self.input, IDENT, self.FOLLOW_IDENT_in_sig_exp_primary1101)

                    id_tree = self._adaptor.createWithPayload(id)
                    self._adaptor.addChild(root_0, id_tree)

                    #action start
                    retval.text = id.text
                    #action end


                elif alt15 == 2:
                    # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/cadlang.g:286:11: LP se= sig_expression RP
                    pass
                    root_0 = self._adaptor.nil()

                    LP27=self.match(self.input, LP, self.FOLLOW_LP_in_sig_exp_primary1125)

                    LP27_tree = self._adaptor.createWithPayload(LP27)
                    self._adaptor.addChild(root_0, LP27_tree)

                    self._state.following.append(self.FOLLOW_sig_expression_in_sig_exp_primary1129)
                    se = self.sig_expression()

                    self._state.following.pop()
                    self._adaptor.addChild(root_0, se.tree)
                    RP28=self.match(self.input, RP, self.FOLLOW_RP_in_sig_exp_primary1131)

                    RP28_tree = self._adaptor.createWithPayload(RP28)
                    self._adaptor.addChild(root_0, RP28_tree)

                    #action start
                    retval.text = '('+((se is not None) and [se.text] or [None])[0]+')'
                    #action end


                retval.stop = self.input.LT(-1)


                retval.tree = self._adaptor.rulePostProcessing(root_0)
                self._adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop)


            except RecognitionException, re:
                self.reportError(re)
                self.recover(self.input, re)
                retval.tree = self._adaptor.errorNode(self.input, retval.start, self.input.LT(-1), re)
        finally:

            pass

        return retval

    # $ANTLR end "sig_exp_primary"

    class bool_exp_return(ParserRuleReturnScope):
        def __init__(self):
            ParserRuleReturnScope.__init__(self)

            self.text = None
            self.tree = None




    # $ANTLR start "bool_exp"
    # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/cadlang.g:290:1: bool_exp returns [text] : (b1= bool_and ( OR b2= bool_and )* | );
    def bool_exp(self, ):

        retval = self.bool_exp_return()
        retval.start = self.input.LT(1)

        root_0 = None

        OR29 = None
        b1 = None

        b2 = None


        OR29_tree = None

        try:
            try:
                # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/cadlang.g:291:12: (b1= bool_and ( OR b2= bool_and )* | )
                alt17 = 2
                LA17_0 = self.input.LA(1)

                if (LA17_0 == IDENT or LA17_0 == LP or (NOT <= LA17_0 <= F)) :
                    alt17 = 1
                elif ((LB <= LA17_0 <= RB) or (DEFAULT <= LA17_0 <= WHEN) or LA17_0 == RP or LA17_0 == COM) :
                    alt17 = 2
                else:
                    nvae = NoViableAltException("", 17, 0, self.input)

                    raise nvae

                if alt17 == 1:
                    # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/cadlang.g:292:13: b1= bool_and ( OR b2= bool_and )*
                    pass
                    root_0 = self._adaptor.nil()

                    self._state.following.append(self.FOLLOW_bool_and_in_bool_exp1225)
                    b1 = self.bool_and()

                    self._state.following.pop()
                    self._adaptor.addChild(root_0, b1.tree)
                    #action start
                    retval.text = ((b1 is not None) and [b1.text] or [None])[0]
                    #action end
                    # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/cadlang.g:293:13: ( OR b2= bool_and )*
                    while True: #loop16
                        alt16 = 2
                        LA16_0 = self.input.LA(1)

                        if (LA16_0 == OR) :
                            alt16 = 1


                        if alt16 == 1:
                            # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/cadlang.g:293:14: OR b2= bool_and
                            pass
                            OR29=self.match(self.input, OR, self.FOLLOW_OR_in_bool_exp1242)

                            OR29_tree = self._adaptor.createWithPayload(OR29)
                            self._adaptor.addChild(root_0, OR29_tree)

                            self._state.following.append(self.FOLLOW_bool_and_in_bool_exp1246)
                            b2 = self.bool_and()

                            self._state.following.pop()
                            self._adaptor.addChild(root_0, b2.tree)
                            #action start
                            retval.text = retval.text + ' '+'or '+((b2 is not None) and [b2.text] or [None])[0]
                            #action end


                        else:
                            break #loop16




                elif alt17 == 2:
                    # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/cadlang.g:294:14:
                    pass
                    root_0 = self._adaptor.nil()

                    #action start
                    retval.text = ""
                    #action end


                retval.stop = self.input.LT(-1)


                retval.tree = self._adaptor.rulePostProcessing(root_0)
                self._adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop)


            except RecognitionException, re:
                self.reportError(re)
                self.recover(self.input, re)
                retval.tree = self._adaptor.errorNode(self.input, retval.start, self.input.LT(-1), re)
        finally:

            pass

        return retval

    # $ANTLR end "bool_exp"

    class bool_and_return(ParserRuleReturnScope):
        def __init__(self):
            ParserRuleReturnScope.__init__(self)

            self.text = None
            self.tree = None




    # $ANTLR start "bool_and"
    # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/cadlang.g:297:1: bool_and returns [text] : b1= bool_primary ( AND b2= bool_primary )* ;
    def bool_and(self, ):

        retval = self.bool_and_return()
        retval.start = self.input.LT(1)

        root_0 = None

        AND30 = None
        b1 = None

        b2 = None


        AND30_tree = None

        try:
            try:
                # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/cadlang.g:298:12: (b1= bool_primary ( AND b2= bool_primary )* )
                # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/cadlang.g:298:14: b1= bool_primary ( AND b2= bool_primary )*
                pass
                root_0 = self._adaptor.nil()

                self._state.following.append(self.FOLLOW_bool_primary_in_bool_and1311)
                b1 = self.bool_primary()

                self._state.following.pop()
                self._adaptor.addChild(root_0, b1.tree)
                #action start
                retval.text = ((b1 is not None) and [b1.text] or [None])[0]
                #action end
                # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/cadlang.g:299:13: ( AND b2= bool_primary )*
                while True: #loop18
                    alt18 = 2
                    LA18_0 = self.input.LA(1)

                    if (LA18_0 == AND) :
                        alt18 = 1


                    if alt18 == 1:
                        # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/cadlang.g:299:14: AND b2= bool_primary
                        pass
                        AND30=self.match(self.input, AND, self.FOLLOW_AND_in_bool_and1328)

                        AND30_tree = self._adaptor.createWithPayload(AND30)
                        self._adaptor.addChild(root_0, AND30_tree)

                        self._state.following.append(self.FOLLOW_bool_primary_in_bool_and1332)
                        b2 = self.bool_primary()

                        self._state.following.pop()
                        self._adaptor.addChild(root_0, b2.tree)
                        #action start
                        retval.text = retval.text + ' '+'and '+((b2 is not None) and [b2.text] or [None])[0]
                        #action end


                    else:
                        break #loop18





                retval.stop = self.input.LT(-1)


                retval.tree = self._adaptor.rulePostProcessing(root_0)
                self._adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop)


            except RecognitionException, re:
                self.reportError(re)
                self.recover(self.input, re)
                retval.tree = self._adaptor.errorNode(self.input, retval.start, self.input.LT(-1), re)
        finally:

            pass

        return retval

    # $ANTLR end "bool_and"

    class bool_primary_return(ParserRuleReturnScope):
        def __init__(self):
            ParserRuleReturnScope.__init__(self)

            self.text = None
            self.tree = None




    # $ANTLR start "bool_primary"
    # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/cadlang.g:302:1: bool_primary returns [text] : ( NOT b1= bool_primary | b2= bool_constant | id= IDENT | LP b3= bool_exp RP );
    def bool_primary(self, ):

        retval = self.bool_primary_return()
        retval.start = self.input.LT(1)

        root_0 = None

        id = None
        NOT31 = None
        LP32 = None
        RP33 = None
        b1 = None

        b2 = None

        b3 = None


        id_tree = None
        NOT31_tree = None
        LP32_tree = None
        RP33_tree = None

        try:
            try:
                # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/cadlang.g:303:12: ( NOT b1= bool_primary | b2= bool_constant | id= IDENT | LP b3= bool_exp RP )
                alt19 = 4
                LA19 = self.input.LA(1)
                if LA19 == NOT:
                    alt19 = 1
                elif LA19 == T or LA19 == F:
                    alt19 = 2
                elif LA19 == IDENT:
                    alt19 = 3
                elif LA19 == LP:
                    alt19 = 4
                else:
                    nvae = NoViableAltException("", 19, 0, self.input)

                    raise nvae

                if alt19 == 1:
                    # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/cadlang.g:303:14: NOT b1= bool_primary
                    pass
                    root_0 = self._adaptor.nil()

                    NOT31=self.match(self.input, NOT, self.FOLLOW_NOT_in_bool_primary1379)

                    NOT31_tree = self._adaptor.createWithPayload(NOT31)
                    self._adaptor.addChild(root_0, NOT31_tree)

                    self._state.following.append(self.FOLLOW_bool_primary_in_bool_primary1383)
                    b1 = self.bool_primary()

                    self._state.following.pop()
                    self._adaptor.addChild(root_0, b1.tree)
                    #action start
                    retval.text = 'not '+((b1 is not None) and [b1.text] or [None])[0]
                    #action end


                elif alt19 == 2:
                    # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/cadlang.g:305:14: b2= bool_constant
                    pass
                    root_0 = self._adaptor.nil()

                    self._state.following.append(self.FOLLOW_bool_constant_in_bool_primary1415)
                    b2 = self.bool_constant()

                    self._state.following.pop()
                    self._adaptor.addChild(root_0, b2.tree)
                    #action start
                    retval.text = ((b2 is not None) and [b2.text] or [None])[0]
                    #action end


                elif alt19 == 3:
                    # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/cadlang.g:307:14: id= IDENT
                    pass
                    root_0 = self._adaptor.nil()

                    id=self.match(self.input, IDENT, self.FOLLOW_IDENT_in_bool_primary1438)

                    id_tree = self._adaptor.createWithPayload(id)
                    self._adaptor.addChild(root_0, id_tree)

                    #action start
                    retval.text = id.text
                    #action end


                elif alt19 == 4:
                    # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/cadlang.g:309:14: LP b3= bool_exp RP
                    pass
                    root_0 = self._adaptor.nil()

                    LP32=self.match(self.input, LP, self.FOLLOW_LP_in_bool_primary1470)

                    LP32_tree = self._adaptor.createWithPayload(LP32)
                    self._adaptor.addChild(root_0, LP32_tree)

                    self._state.following.append(self.FOLLOW_bool_exp_in_bool_primary1474)
                    b3 = self.bool_exp()

                    self._state.following.pop()
                    self._adaptor.addChild(root_0, b3.tree)
                    RP33=self.match(self.input, RP, self.FOLLOW_RP_in_bool_primary1476)

                    RP33_tree = self._adaptor.createWithPayload(RP33)
                    self._adaptor.addChild(root_0, RP33_tree)

                    #action start
                    retval.text = '('+((b3 is not None) and [b3.text] or [None])[0]+')'
                    #action end


                retval.stop = self.input.LT(-1)


                retval.tree = self._adaptor.rulePostProcessing(root_0)
                self._adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop)


            except RecognitionException, re:
                self.reportError(re)
                self.recover(self.input, re)
                retval.tree = self._adaptor.errorNode(self.input, retval.start, self.input.LT(-1), re)
        finally:

            pass

        return retval

    # $ANTLR end "bool_primary"

    class bool_constant_return(ParserRuleReturnScope):
        def __init__(self):
            ParserRuleReturnScope.__init__(self)

            self.text = None
            self.tree = None




    # $ANTLR start "bool_constant"
    # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/cadlang.g:313:1: bool_constant returns [text] : ( T | F );
    def bool_constant(self, ):

        retval = self.bool_constant_return()
        retval.start = self.input.LT(1)

        root_0 = None

        T34 = None
        F35 = None

        T34_tree = None
        F35_tree = None

        try:
            try:
                # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/cadlang.g:314:12: ( T | F )
                alt20 = 2
                LA20_0 = self.input.LA(1)

                if (LA20_0 == T) :
                    alt20 = 1
                elif (LA20_0 == F) :
                    alt20 = 2
                else:
                    nvae = NoViableAltException("", 20, 0, self.input)

                    raise nvae

                if alt20 == 1:
                    # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/cadlang.g:314:14: T
                    pass
                    root_0 = self._adaptor.nil()

                    T34=self.match(self.input, T, self.FOLLOW_T_in_bool_constant1514)

                    T34_tree = self._adaptor.createWithPayload(T34)
                    self._adaptor.addChild(root_0, T34_tree)

                    #action start
                    retval.text = 'true'
                    #action end


                elif alt20 == 2:
                    # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/cadlang.g:315:14: F
                    pass
                    root_0 = self._adaptor.nil()

                    F35=self.match(self.input, F, self.FOLLOW_F_in_bool_constant1532)

                    F35_tree = self._adaptor.createWithPayload(F35)
                    self._adaptor.addChild(root_0, F35_tree)

                    #action start
                    retval.text = 'false'
                    #action end


                retval.stop = self.input.LT(-1)


                retval.tree = self._adaptor.rulePostProcessing(root_0)
                self._adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop)


            except RecognitionException, re:
                self.reportError(re)
                self.recover(self.input, re)
                retval.tree = self._adaptor.errorNode(self.input, retval.start, self.input.LT(-1), re)
        finally:

            pass

        return retval

    # $ANTLR end "bool_constant"

    class constraints_return(ParserRuleReturnScope):
        def __init__(self):
            ParserRuleReturnScope.__init__(self)

            self.text = None
            self.tree = None




    # $ANTLR start "constraints"
    # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/cadlang.g:318:1: constraints returns [text] : CONST t1= const_exp (t2= const_exp )* ENDCONST ;
    def constraints(self, ):

        retval = self.constraints_return()
        retval.start = self.input.LT(1)

        root_0 = None

        CONST36 = None
        ENDCONST37 = None
        t1 = None

        t2 = None


        CONST36_tree = None
        ENDCONST37_tree = None


        SEP = ';'

        try:
            try:
                # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/cadlang.g:322:12: ( CONST t1= const_exp (t2= const_exp )* ENDCONST )
                # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/cadlang.g:322:14: CONST t1= const_exp (t2= const_exp )* ENDCONST
                pass
                root_0 = self._adaptor.nil()

                CONST36=self.match(self.input, CONST, self.FOLLOW_CONST_in_constraints1595)

                CONST36_tree = self._adaptor.createWithPayload(CONST36)
                self._adaptor.addChild(root_0, CONST36_tree)

                self._state.following.append(self.FOLLOW_const_exp_in_constraints1599)
                t1 = self.const_exp()

                self._state.following.pop()
                self._adaptor.addChild(root_0, t1.tree)
                #action start
                retval.text = ((t1 is not None) and [t1.text] or [None])[0]+SEP
                #action end
                # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/cadlang.g:323:14: (t2= const_exp )*
                while True: #loop21
                    alt21 = 2
                    LA21_0 = self.input.LA(1)

                    if ((SYNC <= LA21_0 <= INC)) :
                        alt21 = 1


                    if alt21 == 1:
                        # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/cadlang.g:323:15: t2= const_exp
                        pass
                        self._state.following.append(self.FOLLOW_const_exp_in_constraints1619)
                        t2 = self.const_exp()

                        self._state.following.pop()
                        self._adaptor.addChild(root_0, t2.tree)
                        #action start
                        retval.text = retval.text+'\n'+((t2 is not None) and [t2.text] or [None])[0]+SEP
                        #action end


                    else:
                        break #loop21


                ENDCONST37=self.match(self.input, ENDCONST, self.FOLLOW_ENDCONST_in_constraints1625)

                ENDCONST37_tree = self._adaptor.createWithPayload(ENDCONST37)
                self._adaptor.addChild(root_0, ENDCONST37_tree)




                retval.stop = self.input.LT(-1)


                retval.tree = self._adaptor.rulePostProcessing(root_0)
                self._adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop)


            except RecognitionException, re:
                self.reportError(re)
                self.recover(self.input, re)
                retval.tree = self._adaptor.errorNode(self.input, retval.start, self.input.LT(-1), re)
        finally:

            pass

        return retval

    # $ANTLR end "constraints"

    class const_exp_return(ParserRuleReturnScope):
        def __init__(self):
            ParserRuleReturnScope.__init__(self)

            self.text = None
            self.tree = None




    # $ANTLR start "const_exp"
    # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/cadlang.g:326:1: const_exp returns [text] : ( SYNC LP t1= exp_list RP | EXCL LP t2= exp_list RP | INC LP t3= sig_expression COM t4= sig_expression RP );
    def const_exp(self, ):

        retval = self.const_exp_return()
        retval.start = self.input.LT(1)

        root_0 = None

        SYNC38 = None
        LP39 = None
        RP40 = None
        EXCL41 = None
        LP42 = None
        RP43 = None
        INC44 = None
        LP45 = None
        COM46 = None
        RP47 = None
        t1 = None

        t2 = None

        t3 = None

        t4 = None


        SYNC38_tree = None
        LP39_tree = None
        RP40_tree = None
        EXCL41_tree = None
        LP42_tree = None
        RP43_tree = None
        INC44_tree = None
        LP45_tree = None
        COM46_tree = None
        RP47_tree = None

        try:
            try:
                # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/cadlang.g:327:12: ( SYNC LP t1= exp_list RP | EXCL LP t2= exp_list RP | INC LP t3= sig_expression COM t4= sig_expression RP )
                alt22 = 3
                LA22 = self.input.LA(1)
                if LA22 == SYNC:
                    alt22 = 1
                elif LA22 == EXCL:
                    alt22 = 2
                elif LA22 == INC:
                    alt22 = 3
                else:
                    nvae = NoViableAltException("", 22, 0, self.input)

                    raise nvae

                if alt22 == 1:
                    # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/cadlang.g:327:14: SYNC LP t1= exp_list RP
                    pass
                    root_0 = self._adaptor.nil()

                    SYNC38=self.match(self.input, SYNC, self.FOLLOW_SYNC_in_const_exp1669)

                    SYNC38_tree = self._adaptor.createWithPayload(SYNC38)
                    self._adaptor.addChild(root_0, SYNC38_tree)

                    LP39=self.match(self.input, LP, self.FOLLOW_LP_in_const_exp1671)

                    LP39_tree = self._adaptor.createWithPayload(LP39)
                    self._adaptor.addChild(root_0, LP39_tree)

                    self._state.following.append(self.FOLLOW_exp_list_in_const_exp1675)
                    t1 = self.exp_list()

                    self._state.following.pop()
                    self._adaptor.addChild(root_0, t1.tree)
                    RP40=self.match(self.input, RP, self.FOLLOW_RP_in_const_exp1677)

                    RP40_tree = self._adaptor.createWithPayload(RP40)
                    self._adaptor.addChild(root_0, RP40_tree)

                    #action start
                    retval.text = 'synchro('+((t1 is not None) and [t1.text] or [None])[0]+')'
                    #action end


                elif alt22 == 2:
                    # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/cadlang.g:329:14: EXCL LP t2= exp_list RP
                    pass
                    root_0 = self._adaptor.nil()

                    EXCL41=self.match(self.input, EXCL, self.FOLLOW_EXCL_in_const_exp1707)

                    EXCL41_tree = self._adaptor.createWithPayload(EXCL41)
                    self._adaptor.addChild(root_0, EXCL41_tree)

                    LP42=self.match(self.input, LP, self.FOLLOW_LP_in_const_exp1709)

                    LP42_tree = self._adaptor.createWithPayload(LP42)
                    self._adaptor.addChild(root_0, LP42_tree)

                    self._state.following.append(self.FOLLOW_exp_list_in_const_exp1713)
                    t2 = self.exp_list()

                    self._state.following.pop()
                    self._adaptor.addChild(root_0, t2.tree)
                    RP43=self.match(self.input, RP, self.FOLLOW_RP_in_const_exp1715)

                    RP43_tree = self._adaptor.createWithPayload(RP43)
                    self._adaptor.addChild(root_0, RP43_tree)

                    #action start
                    retval.text = 'exclus('+((t2 is not None) and [t2.text] or [None])[0]+')'
                    #action end


                elif alt22 == 3:
                    # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/cadlang.g:331:14: INC LP t3= sig_expression COM t4= sig_expression RP
                    pass
                    root_0 = self._adaptor.nil()

                    INC44=self.match(self.input, INC, self.FOLLOW_INC_in_const_exp1745)

                    INC44_tree = self._adaptor.createWithPayload(INC44)
                    self._adaptor.addChild(root_0, INC44_tree)

                    LP45=self.match(self.input, LP, self.FOLLOW_LP_in_const_exp1747)

                    LP45_tree = self._adaptor.createWithPayload(LP45)
                    self._adaptor.addChild(root_0, LP45_tree)

                    self._state.following.append(self.FOLLOW_sig_expression_in_const_exp1751)
                    t3 = self.sig_expression()

                    self._state.following.pop()
                    self._adaptor.addChild(root_0, t3.tree)
                    COM46=self.match(self.input, COM, self.FOLLOW_COM_in_const_exp1753)

                    COM46_tree = self._adaptor.createWithPayload(COM46)
                    self._adaptor.addChild(root_0, COM46_tree)

                    self._state.following.append(self.FOLLOW_sig_expression_in_const_exp1757)
                    t4 = self.sig_expression()

                    self._state.following.pop()
                    self._adaptor.addChild(root_0, t4.tree)
                    RP47=self.match(self.input, RP, self.FOLLOW_RP_in_const_exp1759)

                    RP47_tree = self._adaptor.createWithPayload(RP47)
                    self._adaptor.addChild(root_0, RP47_tree)

                    #action start
                    retval.text = 'included('+((t3 is not None) and [t3.text] or [None])[0]+', '+ ((t4 is not None) and [t4.text] or [None])[0]+')'
                    #action end


                retval.stop = self.input.LT(-1)


                retval.tree = self._adaptor.rulePostProcessing(root_0)
                self._adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop)


            except RecognitionException, re:
                self.reportError(re)
                self.recover(self.input, re)
                retval.tree = self._adaptor.errorNode(self.input, retval.start, self.input.LT(-1), re)
        finally:

            pass

        return retval

    # $ANTLR end "const_exp"

    class exp_list_return(ParserRuleReturnScope):
        def __init__(self):
            ParserRuleReturnScope.__init__(self)

            self.text = None
            self.tree = None




    # $ANTLR start "exp_list"
    # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/cadlang.g:335:1: exp_list returns [text] : t1= sig_expression ( COM t2= sig_expression )* ;
    def exp_list(self, ):

        retval = self.exp_list_return()
        retval.start = self.input.LT(1)

        root_0 = None

        COM48 = None
        t1 = None

        t2 = None


        COM48_tree = None

        try:
            try:
                # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/cadlang.g:336:12: (t1= sig_expression ( COM t2= sig_expression )* )
                # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/cadlang.g:336:14: t1= sig_expression ( COM t2= sig_expression )*
                pass
                root_0 = self._adaptor.nil()

                self._state.following.append(self.FOLLOW_sig_expression_in_exp_list1823)
                t1 = self.sig_expression()

                self._state.following.pop()
                self._adaptor.addChild(root_0, t1.tree)
                #action start
                retval.text = ((t1 is not None) and [t1.text] or [None])[0]
                #action end
                # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/cadlang.g:337:12: ( COM t2= sig_expression )*
                while True: #loop23
                    alt23 = 2
                    LA23_0 = self.input.LA(1)

                    if (LA23_0 == COM) :
                        alt23 = 1


                    if alt23 == 1:
                        # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/cadlang.g:337:13: COM t2= sig_expression
                        pass
                        COM48=self.match(self.input, COM, self.FOLLOW_COM_in_exp_list1839)

                        COM48_tree = self._adaptor.createWithPayload(COM48)
                        self._adaptor.addChild(root_0, COM48_tree)

                        self._state.following.append(self.FOLLOW_sig_expression_in_exp_list1843)
                        t2 = self.sig_expression()

                        self._state.following.pop()
                        self._adaptor.addChild(root_0, t2.tree)
                        #action start
                        retval.text = retval.text +', '+((t2 is not None) and [t2.text] or [None])[0]
                        #action end


                    else:
                        break #loop23





                retval.stop = self.input.LT(-1)


                retval.tree = self._adaptor.rulePostProcessing(root_0)
                self._adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop)


            except RecognitionException, re:
                self.reportError(re)
                self.recover(self.input, re)
                retval.tree = self._adaptor.errorNode(self.input, retval.start, self.input.LT(-1), re)
        finally:

            pass

        return retval

    # $ANTLR end "exp_list"


    # Delegated rules




    FOLLOW_NAME_in_cad_model139 = frozenset([5])
    FOLLOW_IDENT_in_cad_model143 = frozenset([5, 7, 13])
    FOLLOW_macro_in_cad_model162 = frozenset([5, 7, 13, 24])
    FOLLOW_transition_in_cad_model164 = frozenset([5, 7, 13, 24])
    FOLLOW_dec_in_cad_model166 = frozenset([5, 7, 13, 24])
    FOLLOW_constraints_in_cad_model203 = frozenset([])
    FOLLOW_EOF_in_cad_model222 = frozenset([1])
    FOLLOW_IDENT_in_dec270 = frozenset([6, 8, 9])
    FOLLOW_modifier_in_dec275 = frozenset([6])
    FOLLOW_SC_in_dec281 = frozenset([1])
    FOLLOW_IDENT_in_transition340 = frozenset([7, 8, 9])
    FOLLOW_modifier_in_transition345 = frozenset([7])
    FOLLOW_TARROW_in_transition349 = frozenset([5])
    FOLLOW_IDENT_in_transition353 = frozenset([6])
    FOLLOW_SC_in_transition355 = frozenset([5, 10, 17])
    FOLLOW_guard_in_transition359 = frozenset([6, 8, 9, 12])
    FOLLOW_modifier_in_transition364 = frozenset([6, 12])
    FOLLOW_note_in_transition370 = frozenset([6])
    FOLLOW_SC_in_transition372 = frozenset([1])
    FOLLOW_IDENT_in_transition416 = frozenset([7])
    FOLLOW_TARROW_in_transition418 = frozenset([6])
    FOLLOW_SC_in_transition421 = frozenset([5, 10, 17])
    FOLLOW_guard_in_transition425 = frozenset([6, 12])
    FOLLOW_note_in_transition429 = frozenset([6])
    FOLLOW_SC_in_transition431 = frozenset([1])
    FOLLOW_TARROW_in_transition474 = frozenset([5])
    FOLLOW_IDENT_in_transition478 = frozenset([6])
    FOLLOW_SC_in_transition481 = frozenset([1, 6, 12])
    FOLLOW_note_in_transition486 = frozenset([6])
    FOLLOW_SC_in_transition488 = frozenset([1])
    FOLLOW_set_in_modifier0 = frozenset([1])
    FOLLOW_sig_expression_in_guard634 = frozenset([10])
    FOLLOW_LB_in_guard638 = frozenset([5, 11, 17, 21, 22, 23])
    FOLLOW_bool_exp_in_guard643 = frozenset([11])
    FOLLOW_RB_in_guard646 = frozenset([1, 5])
    FOLLOW_IDENT_in_guard651 = frozenset([1])
    FOLLOW_TEXT_in_note717 = frozenset([1])
    FOLLOW_MACRO_in_macro795 = frozenset([5])
    FOLLOW_IDENT_in_macro799 = frozenset([5, 7, 13, 14])
    FOLLOW_macro_in_macro829 = frozenset([5, 7, 13, 14])
    FOLLOW_transition_in_macro831 = frozenset([5, 7, 13, 14])
    FOLLOW_dec_in_macro833 = frozenset([5, 7, 13, 14])
    FOLLOW_ENDMACRO_in_macro849 = frozenset([1])
    FOLLOW_sig_exp_in_sig_expression923 = frozenset([1, 15])
    FOLLOW_DEFAULT_in_sig_expression939 = frozenset([5, 17])
    FOLLOW_sig_exp_in_sig_expression944 = frozenset([1, 15])
    FOLLOW_sig_exp_primary_in_sig_exp1014 = frozenset([1, 16])
    FOLLOW_WHEN_in_sig_exp1029 = frozenset([5, 16, 17, 21, 22, 23])
    FOLLOW_bool_exp_in_sig_exp1034 = frozenset([1, 16])
    FOLLOW_IDENT_in_sig_exp_primary1101 = frozenset([1])
    FOLLOW_LP_in_sig_exp_primary1125 = frozenset([5, 17])
    FOLLOW_sig_expression_in_sig_exp_primary1129 = frozenset([18])
    FOLLOW_RP_in_sig_exp_primary1131 = frozenset([1])
    FOLLOW_bool_and_in_bool_exp1225 = frozenset([1, 19])
    FOLLOW_OR_in_bool_exp1242 = frozenset([5, 17, 21, 22, 23])
    FOLLOW_bool_and_in_bool_exp1246 = frozenset([1, 19])
    FOLLOW_bool_primary_in_bool_and1311 = frozenset([1, 20])
    FOLLOW_AND_in_bool_and1328 = frozenset([5, 17, 21, 22, 23])
    FOLLOW_bool_primary_in_bool_and1332 = frozenset([1, 20])
    FOLLOW_NOT_in_bool_primary1379 = frozenset([5, 17, 21, 22, 23])
    FOLLOW_bool_primary_in_bool_primary1383 = frozenset([1])
    FOLLOW_bool_constant_in_bool_primary1415 = frozenset([1])
    FOLLOW_IDENT_in_bool_primary1438 = frozenset([1])
    FOLLOW_LP_in_bool_primary1470 = frozenset([5, 17, 18, 21, 22, 23])
    FOLLOW_bool_exp_in_bool_primary1474 = frozenset([18])
    FOLLOW_RP_in_bool_primary1476 = frozenset([1])
    FOLLOW_T_in_bool_constant1514 = frozenset([1])
    FOLLOW_F_in_bool_constant1532 = frozenset([1])
    FOLLOW_CONST_in_constraints1595 = frozenset([26, 27, 28])
    FOLLOW_const_exp_in_constraints1599 = frozenset([25, 26, 27, 28])
    FOLLOW_const_exp_in_constraints1619 = frozenset([25, 26, 27, 28])
    FOLLOW_ENDCONST_in_constraints1625 = frozenset([1])
    FOLLOW_SYNC_in_const_exp1669 = frozenset([17])
    FOLLOW_LP_in_const_exp1671 = frozenset([5, 17])
    FOLLOW_exp_list_in_const_exp1675 = frozenset([18])
    FOLLOW_RP_in_const_exp1677 = frozenset([1])
    FOLLOW_EXCL_in_const_exp1707 = frozenset([17])
    FOLLOW_LP_in_const_exp1709 = frozenset([5, 17])
    FOLLOW_exp_list_in_const_exp1713 = frozenset([18])
    FOLLOW_RP_in_const_exp1715 = frozenset([1])
    FOLLOW_INC_in_const_exp1745 = frozenset([17])
    FOLLOW_LP_in_const_exp1747 = frozenset([5, 17])
    FOLLOW_sig_expression_in_const_exp1751 = frozenset([29])
    FOLLOW_COM_in_const_exp1753 = frozenset([5, 17])
    FOLLOW_sig_expression_in_const_exp1757 = frozenset([18])
    FOLLOW_RP_in_const_exp1759 = frozenset([1])
    FOLLOW_sig_expression_in_exp_list1823 = frozenset([1, 29])
    FOLLOW_COM_in_exp_list1839 = frozenset([5, 17])
    FOLLOW_sig_expression_in_exp_list1843 = frozenset([1, 29])



def main(argv, stdin=sys.stdin, stdout=sys.stdout, stderr=sys.stderr):
    from antlr3.main import ParserMain
    main = ParserMain("cadlangLexer", cadlangParser)
    main.stdin = stdin
    main.stdout = stdout
    main.stderr = stderr
    main.execute(argv)


if __name__ == '__main__':
    main(sys.argv)



## Filename    : gt_visitors.py
## Author(s)   : Michel Le Borgne
## Created     : 01/2012
## Revision    : 8/12 suppression des macros
## Source      :
##
## Copyright 2010-2012 : IRISA/IRSET
##
## This library is free software; you can redistribute it and/or modify it
## under the terms of the GNU General Public License as published
## by the Free Software Foundation; either version 2.1 of the License, or
## any later version.
##
## This library is distributed in the hope that it will be useful, but
## WITHOUT ANY WARRANTY, WITHOUT EVEN THE IMPLIED WARRANTY OF
## MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.  The software and
## documentation provided here under is on an "as is" basis, and IRISA has
## no obligations to provide maintenance, support, updates, enhancements
## or modifications.
## In no event shall IRISA be liable to any party for direct, indirect,
## special, incidental or consequential damages, including lost profits,
## arising out of the use of this software and its documentation, even if
## IRISA have been advised of the possibility of such damage.  See
## the GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this library; if not, write to the Free Software Foundation,
## Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.
##
## The original code contained here was initially developed by:
##
##     Michel Le Borgne.
##     IRISA
##     Symbiose team
##     IRISA  Campus de Beaulieu
##     35042 RENNES Cedex, FRANCE
##
##
## Contributor(s): Geoffroy Andrieux
##
"""
Visitor for simulator generation
"""
import sys

from cadbiom.models.guard_transitions.simulator.chart_simul_elem import Place, \
                   STransition, Event, Condition, InputPlace, InputEvent
from cadbiom.models.biosignal.sig_expr import SigIdentExpr
from cadbiom.models.biosignal.translators.gt_visitors import compile_cond, compile_event

class GenSimVisitor(object):
    """
    Generate a simulator model - needs a symbol table of the charter model
    """
    def __init__(self, tab_symb, mod, free_clocks, report):
        """
        This visitor fill a simulation object - identifiers must be in
        tab_symb if free_clocks option = False. When free_clocks option = True
        undeclared ids in an event expression are considered as free clocks.
        @param tab_symb: symbol table obtained from TableVisitor
        @param mod: empty simulation model to be built
        @param free_clocks: allows free clocks if True
        @param report: error reporter of CompilReporter type
        """
        self.sim_model = mod       # empty simulator object
        self.tab_symb = tab_symb   # symbol table of chart_model representation
        # symbol table to retrieve simulator model element
        self.mod_symb_tab = dict()
        self.error_reporter = report
        self.error = False
        self.free_clock_collect = free_clocks
        self.free_clocks = [] # collect free clocks


    def visit_chart_model(self, model):
        """
        visit chart model, generate simulation model and collect free clocks
        """
        model.get_root().accept(self)
        # transform free clocks into inputs
        if self.free_clock_collect:
            for efc in self.free_clocks:
                try:
                    evt = self.mod_symb_tab[efc]
                except:
                    evt = InputEvent(SigIdentExpr(efc))
                    self.mod_symb_tab[efc] = evt
                    self.sim_model.add_event(evt, efc)
                    self.sim_model.add_input(evt)

    def visit_cstart_node(self, node):
        """
        do nothing: see transitions
        """
        return

    def visit_ctrap_node(self, node):
        """
        do nothing: see transitions
        """
        return

    def visit_csimple_node(self, node):
        """
        create a place in the simulator model
        register it in the symbol table
        """
        place = Place(node)
        name = node.name
        self.sim_model.add_state_place(place, name)
        self.mod_symb_tab[name] = place
        return

    def visit_cinput_node(self, node):
        """
        create an input place (artificial perm place)
        and an event representing the input
        """
        place = InputPlace(node)
        self.sim_model.add_place(place)
        name = input_perm_name(node)
        self.mod_symb_tab[name] = place
        # add corresponding event with name the name of the input place
        try:
            evt = self.mod_symb_tab[node.name]
        except:
            evt = InputEvent(SigIdentExpr(node.name))
            self.mod_symb_tab[node.name] = evt
            self.sim_model.add_event(evt, node.name)
            self.sim_model.add_input(evt)
        evt.add_ev_place(place)
        return

    def visit_cperm_node(self, node):
        """
        create a place in the simulator model
        register it in the symbol table
        mark it as permanent
        """
        place = Place(node)
        place.set_perm()
        name = node.name
        self.sim_model.add_state_place(place, name)
        self.mod_symb_tab[name] = place
        return

    def visit_cmacro_node(self, node):
        """
        macro nodes are treated as simple nodes (no real macros allowed)
        """
        if len(node.sub_nodes) >0:
            self.error_reporter.display("Macro node "+node.name+" is not empty")
            self.error = True
        self.visit_csimple_node(node)
        return

    def visit_ctop_node(self, node):
        """
        trap node is unique for all the system
        it is created here
        """
        trap_all = Place(None, '_trap_')
        self.mod_symb_tab["__trap_all__"] = trap_all
        self.sim_model.add_place(trap_all)
        self.sim_model.trap = trap_all

        # sub nodes creation (all places are created by this action)
        for snode in node.sub_nodes:
            snode.accept(self)

        # transitions
        for gtr in node.transitions:
            for trans in gtr:
                trans.accept(self)
        return

    def visit_ctransition(self, trans):
        """
        main visitor - the code generation is dispatched in
        several helper methods
        """
        if trans.ori.is_start(): # transition from start
            # nothing to do (start is a frontier mark)
            return
        elif trans.ori.is_input(): # transition from input
            self.build_input_transition(trans)
        elif trans.ext.is_trap(): # transition to trap node
            self.build_transition(trans, True)
        else:  # standard transition
            self.build_transition(trans)
        return



    # auxiliary functions for compilation
    def build_transition(self, trans, trap = False):
        """
        build a transition from a graph transition
        @param trans: the chart transition
        @param trap: True if target is a trap node
        """
        model = self.sim_model
        tab_symb = self.tab_symb
        report = self.error_reporter
        report.set_context(trans.ori.name+'->'+trans.ext.name)
        try:
            ori = self.mod_symb_tab[trans.ori.name]
            if trap:
                ext = self.sim_model.trap
            else:
                ext = self.mod_symb_tab[trans.ext.name]
        except LookupError, exc:
            print exc
            print trans.ori.name +' -> '+trans.ext.name , trap
            print self.mod_symb_tab
            print 'build_transition: BUG!'
            sys.exit(1)
        trs = STransition(ori, ext, trans)
        self.mod_symb_tab[trans.get_key()] = trs
        # event
        if len(trans.event)>0:
            event_text = trans.event
            event_tree, st_ev, fcl = compile_event(event_text, tab_symb,
                                                  self.free_clock_collect,
                                                  report)
            if self.free_clock_collect:
                self.free_clocks.extend(fcl)
            evt = Event(event_tree)
            trs.set_event(evt)
            # see if state events where used
            for ev_n in st_ev:
                try:
                    evt = self.mod_symb_tab[ev_n]
                except:
                    evt = Event(SigIdentExpr(ev_n))
                    self.mod_symb_tab[ev_n] = evt
                    self.sim_model.add_event(evt, ev_n)
                state_name = ev_n[:-1]   # ev_n = state> or ev_n = state<
                sign = ev_n[-1]
                place = self.mod_symb_tab[state_name]
                if sign == '>':
                    place.ev_up = evt
                else:
                    place.ev_down = evt
        # condition
        if len(trans.condition)>0:
            cond_text = trans.condition
            cond_tree = compile_cond(cond_text, tab_symb, report)
            cond = Condition(cond_tree)
            trs.set_condition(cond)
        # action (signal)
        if len(trans.action)>0:
            act_text = trans.action
            act_tree, ste, fcl = compile_event(act_text, tab_symb,
                                               self.free_clock_collect, report)
            if self.free_clock_collect:
                self.free_clocks.extend(fcl)
            if act_tree.is_ident():
                # create the event if not already done
                try:
                    evt = self.mod_symb_tab[act_tree.name]
                except:
                    evt = Event(act_tree)
                    self.mod_symb_tab[act_tree.name] = evt
                    model.add_event(evt, act_tree.name)
                trs.set_signal(evt)
            else:
                report.display("Syntax: Emitted event must be a name ")
                self.error = True
        self.sim_model.add_transition(trs)


    def build_input_transition(self, trans):
        """
        Build a perm place (if not already done) and a transition from
        the perm node to the target of trans.
        @param trans: the chart model transition
        """
        model = self.sim_model
        tab_symb = self.tab_symb
        report = self.error_reporter
        ev_name = trans.ori.name

        # input transition must have no event
        if trans.event:
            mess = "Syntax: A start transition must have no event: ->"
            mess = mess + trans.ext.name
            report.display(mess)
            self.error = True
        perm_name = input_perm_name(trans.ori)
        try:
            # should be created with other places
            input_place = self.mod_symb_tab[perm_name]
        except:
            print 'BUG: input place not built'
            sys.exit(1)

        ext = self.mod_symb_tab[trans.ext.name]
        # new transition from perm place to input target
        trs = STransition(input_place, ext, trans)
        model.add_transition(trs)
        self.mod_symb_tab[input_trans_name(trans)] = trs
        # event - is the input
        try:
            evt = self.mod_symb_tab[ev_name]
        except:
            pass #TODO internal error
        trs.set_event(evt)
        # condition
        if len(trans.condition)>0:
            cond_text = trans.condition
            cond_tree = compile_cond(cond_text, tab_symb, report)
            cond = Condition(cond_tree)
            trs.set_condition(cond)
        # action
        if len(trans.action)>0:
            act_text = trans.action
            # cannot find free clocks in action
            act_tree, ste, fcl = compile_event(act_text, tab_symb, False,
                                               report)
            if act_tree.is_ident():
                # create the event if not already done
                try:
                    evt = self.mod_symb_tab[act_tree.name]
                except:
                    evt = Event(act_tree)
                    self.mod_symb_tab[act_tree.name] = evt
                    model.add_event(evt, act_tree.name)
                trs.set_signal(evt)
            else:
                mess = "Syntax: Emitted event must be an id "
                mess = mess + trans.ori.name + "->" + trans.ext.name
                report.display(mess)
                self.error = True



# helper functions
def input_perm_name(in_node):
    """
    generate a standard name for pseudo perm places associated with inputs
    """
    return "__"+in_node.name+"$$input"

def input_trans_name(trans):
    """
    generate a standard name for transitions
    """
    return '__'+trans.ori.name+"__"+trans.ext.name




# $ANTLR 3.1.2 /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/cadlang.g 2012-09-12 14:02:23

import sys
from cadbiom.antlr3 import *
from cadbiom.antlr3.compat import set, frozenset


# for convenience in actions
HIDDEN = BaseRecognizer.HIDDEN

# token types
ENDCONST=25
F=23
RB=11
SYNC=26
RP=18
LETTER=32
CONST=24
DEFAULT=15
LP=17
INPUT=9
T=22
ENDMACRO=14
EXCL=27
NOT=21
TEXT=12
AND=20
EOF=-1
SC=6
INC=28
NAME=4
WS=30
WHEN=16
COM=29
OR=19
TARROW=7
IDENT=5
LB=10
DIGIT=33
COMMENT=31
MACRO=13
PERM=8


class cadlangLexer(Lexer):

    grammarFileName = "/home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/cadlang.g"
    antlr_version = version_str_to_tuple("3.1.2")
    antlr_version_str = "3.1.2"

    def __init__(self, input=None, state=None):
        if state is None:
            state = RecognizerSharedState()
        Lexer.__init__(self, input, state)

        self.dfa4 = self.DFA4(
            self, 4,
            eot = self.DFA4_eot,
            eof = self.DFA4_eof,
            min = self.DFA4_min,
            max = self.DFA4_max,
            accept = self.DFA4_accept,
            special = self.DFA4_special,
            transition = self.DFA4_transition
            )





    def set_error_reporter(self, err):
        self.error_reporter = err

    def displayRecognitionError(self, tokenNames, re):
        hdr = self.getErrorHeader(re)
        msg = self.getErrorMessage(re, tokenNames)
        self.error_reporter.display(hdr+' '+msg)


    def displayExceptionMessage(self, e):
        msg = self.getErrorMessage(self, e, tokenNames)
        self.error_reporter.display(msg)



    # $ANTLR start "WS"
    def mWS(self, ):

        try:
            _type = WS
            _channel = DEFAULT_CHANNEL

            # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/cadlang.g:345:13: ( ( ' ' | '\\t' | '\\n' ) )
            # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/cadlang.g:345:16: ( ' ' | '\\t' | '\\n' )
            pass
            if (9 <= self.input.LA(1) <= 10) or self.input.LA(1) == 32:
                self.input.consume()
            else:
                mse = MismatchedSetException(None, self.input)
                self.recover(mse)
                raise mse

            #action start
            _channel = HIDDEN;
            #action end



            self._state.type = _type
            self._state.channel = _channel

        finally:

            pass

    # $ANTLR end "WS"



    # $ANTLR start "COMMENT"
    def mCOMMENT(self, ):

        try:
            _type = COMMENT
            _channel = DEFAULT_CHANNEL

            # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/cadlang.g:347:13: ( '//' (~ '\\n' )* '\\n' )
            # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/cadlang.g:347:15: '//' (~ '\\n' )* '\\n'
            pass
            self.match("//")
            # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/cadlang.g:347:19: (~ '\\n' )*
            while True: #loop1
                alt1 = 2
                LA1_0 = self.input.LA(1)

                if ((0 <= LA1_0 <= 9) or (11 <= LA1_0 <= 65535)) :
                    alt1 = 1


                if alt1 == 1:
                    # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/cadlang.g:347:20: ~ '\\n'
                    pass
                    if (0 <= self.input.LA(1) <= 9) or (11 <= self.input.LA(1) <= 65535):
                        self.input.consume()
                    else:
                        mse = MismatchedSetException(None, self.input)
                        self.recover(mse)
                        raise mse



                else:
                    break #loop1


            self.match(10)
            #action start
            _channel = HIDDEN;
            #action end



            self._state.type = _type
            self._state.channel = _channel

        finally:

            pass

    # $ANTLR end "COMMENT"



    # $ANTLR start "AND"
    def mAND(self, ):

        try:
            _type = AND
            _channel = DEFAULT_CHANNEL

            # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/cadlang.g:350:9: ( 'and' )
            # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/cadlang.g:350:11: 'and'
            pass
            self.match("and")



            self._state.type = _type
            self._state.channel = _channel

        finally:

            pass

    # $ANTLR end "AND"



    # $ANTLR start "OR"
    def mOR(self, ):

        try:
            _type = OR
            _channel = DEFAULT_CHANNEL

            # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/cadlang.g:351:9: ( 'or' )
            # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/cadlang.g:351:11: 'or'
            pass
            self.match("or")



            self._state.type = _type
            self._state.channel = _channel

        finally:

            pass

    # $ANTLR end "OR"



    # $ANTLR start "NOT"
    def mNOT(self, ):

        try:
            _type = NOT
            _channel = DEFAULT_CHANNEL

            # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/cadlang.g:352:9: ( 'not' )
            # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/cadlang.g:352:11: 'not'
            pass
            self.match("not")



            self._state.type = _type
            self._state.channel = _channel

        finally:

            pass

    # $ANTLR end "NOT"



    # $ANTLR start "T"
    def mT(self, ):

        try:
            _type = T
            _channel = DEFAULT_CHANNEL

            # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/cadlang.g:353:9: ( 'true' )
            # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/cadlang.g:353:11: 'true'
            pass
            self.match("true")



            self._state.type = _type
            self._state.channel = _channel

        finally:

            pass

    # $ANTLR end "T"



    # $ANTLR start "F"
    def mF(self, ):

        try:
            _type = F
            _channel = DEFAULT_CHANNEL

            # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/cadlang.g:354:9: ( 'false' )
            # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/cadlang.g:354:11: 'false'
            pass
            self.match("false")



            self._state.type = _type
            self._state.channel = _channel

        finally:

            pass

    # $ANTLR end "F"



    # $ANTLR start "LP"
    def mLP(self, ):

        try:
            _type = LP
            _channel = DEFAULT_CHANNEL

            # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/cadlang.g:356:9: ( '(' )
            # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/cadlang.g:356:11: '('
            pass
            self.match(40)



            self._state.type = _type
            self._state.channel = _channel

        finally:

            pass

    # $ANTLR end "LP"



    # $ANTLR start "RP"
    def mRP(self, ):

        try:
            _type = RP
            _channel = DEFAULT_CHANNEL

            # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/cadlang.g:357:9: ( ')' )
            # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/cadlang.g:357:11: ')'
            pass
            self.match(41)



            self._state.type = _type
            self._state.channel = _channel

        finally:

            pass

    # $ANTLR end "RP"



    # $ANTLR start "LB"
    def mLB(self, ):

        try:
            _type = LB
            _channel = DEFAULT_CHANNEL

            # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/cadlang.g:358:9: ( '[' )
            # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/cadlang.g:358:11: '['
            pass
            self.match(91)



            self._state.type = _type
            self._state.channel = _channel

        finally:

            pass

    # $ANTLR end "LB"



    # $ANTLR start "RB"
    def mRB(self, ):

        try:
            _type = RB
            _channel = DEFAULT_CHANNEL

            # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/cadlang.g:359:9: ( ']' )
            # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/cadlang.g:359:11: ']'
            pass
            self.match(93)



            self._state.type = _type
            self._state.channel = _channel

        finally:

            pass

    # $ANTLR end "RB"



    # $ANTLR start "SC"
    def mSC(self, ):

        try:
            _type = SC
            _channel = DEFAULT_CHANNEL

            # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/cadlang.g:360:9: ( ';' )
            # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/cadlang.g:360:11: ';'
            pass
            self.match(59)



            self._state.type = _type
            self._state.channel = _channel

        finally:

            pass

    # $ANTLR end "SC"



    # $ANTLR start "COM"
    def mCOM(self, ):

        try:
            _type = COM
            _channel = DEFAULT_CHANNEL

            # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/cadlang.g:361:9: ( ',' )
            # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/cadlang.g:361:11: ','
            pass
            self.match(44)



            self._state.type = _type
            self._state.channel = _channel

        finally:

            pass

    # $ANTLR end "COM"



    # $ANTLR start "TARROW"
    def mTARROW(self, ):

        try:
            _type = TARROW
            _channel = DEFAULT_CHANNEL

            # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/cadlang.g:364:9: ( '-->' )
            # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/cadlang.g:364:11: '-->'
            pass
            self.match("-->")



            self._state.type = _type
            self._state.channel = _channel

        finally:

            pass

    # $ANTLR end "TARROW"



    # $ANTLR start "PERM"
    def mPERM(self, ):

        try:
            _type = PERM
            _channel = DEFAULT_CHANNEL

            # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/cadlang.g:365:9: ( '/p' )
            # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/cadlang.g:365:11: '/p'
            pass
            self.match("/p")



            self._state.type = _type
            self._state.channel = _channel

        finally:

            pass

    # $ANTLR end "PERM"



    # $ANTLR start "INPUT"
    def mINPUT(self, ):

        try:
            _type = INPUT
            _channel = DEFAULT_CHANNEL

            # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/cadlang.g:366:9: ( '/i' )
            # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/cadlang.g:366:11: '/i'
            pass
            self.match("/i")



            self._state.type = _type
            self._state.channel = _channel

        finally:

            pass

    # $ANTLR end "INPUT"



    # $ANTLR start "MACRO"
    def mMACRO(self, ):

        try:
            _type = MACRO
            _channel = DEFAULT_CHANNEL

            # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/cadlang.g:367:9: ( '/macro' )
            # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/cadlang.g:367:11: '/macro'
            pass
            self.match("/macro")



            self._state.type = _type
            self._state.channel = _channel

        finally:

            pass

    # $ANTLR end "MACRO"



    # $ANTLR start "ENDMACRO"
    def mENDMACRO(self, ):

        try:
            _type = ENDMACRO
            _channel = DEFAULT_CHANNEL

            # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/cadlang.g:368:9: ( '/endmacro' )
            # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/cadlang.g:368:11: '/endmacro'
            pass
            self.match("/endmacro")



            self._state.type = _type
            self._state.channel = _channel

        finally:

            pass

    # $ANTLR end "ENDMACRO"



    # $ANTLR start "NAME"
    def mNAME(self, ):

        try:
            _type = NAME
            _channel = DEFAULT_CHANNEL

            # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/cadlang.g:369:9: ( '/name' )
            # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/cadlang.g:369:11: '/name'
            pass
            self.match("/name")



            self._state.type = _type
            self._state.channel = _channel

        finally:

            pass

    # $ANTLR end "NAME"



    # $ANTLR start "DEFAULT"
    def mDEFAULT(self, ):

        try:
            _type = DEFAULT
            _channel = DEFAULT_CHANNEL

            # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/cadlang.g:370:9: ( 'default' )
            # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/cadlang.g:370:11: 'default'
            pass
            self.match("default")



            self._state.type = _type
            self._state.channel = _channel

        finally:

            pass

    # $ANTLR end "DEFAULT"



    # $ANTLR start "WHEN"
    def mWHEN(self, ):

        try:
            _type = WHEN
            _channel = DEFAULT_CHANNEL

            # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/cadlang.g:371:9: ( 'when' )
            # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/cadlang.g:371:11: 'when'
            pass
            self.match("when")



            self._state.type = _type
            self._state.channel = _channel

        finally:

            pass

    # $ANTLR end "WHEN"



    # $ANTLR start "CONST"
    def mCONST(self, ):

        try:
            _type = CONST
            _channel = DEFAULT_CHANNEL

            # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/cadlang.g:372:9: ( '/constraints' )
            # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/cadlang.g:372:11: '/constraints'
            pass
            self.match("/constraints")



            self._state.type = _type
            self._state.channel = _channel

        finally:

            pass

    # $ANTLR end "CONST"



    # $ANTLR start "ENDCONST"
    def mENDCONST(self, ):

        try:
            _type = ENDCONST
            _channel = DEFAULT_CHANNEL

            # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/cadlang.g:373:9: ( '/endconstraints' )
            # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/cadlang.g:373:11: '/endconstraints'
            pass
            self.match("/endconstraints")



            self._state.type = _type
            self._state.channel = _channel

        finally:

            pass

    # $ANTLR end "ENDCONST"



    # $ANTLR start "SYNC"
    def mSYNC(self, ):

        try:
            _type = SYNC
            _channel = DEFAULT_CHANNEL

            # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/cadlang.g:374:9: ( 'synchro' )
            # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/cadlang.g:374:11: 'synchro'
            pass
            self.match("synchro")



            self._state.type = _type
            self._state.channel = _channel

        finally:

            pass

    # $ANTLR end "SYNC"



    # $ANTLR start "EXCL"
    def mEXCL(self, ):

        try:
            _type = EXCL
            _channel = DEFAULT_CHANNEL

            # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/cadlang.g:375:9: ( 'exclus' )
            # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/cadlang.g:375:11: 'exclus'
            pass
            self.match("exclus")



            self._state.type = _type
            self._state.channel = _channel

        finally:

            pass

    # $ANTLR end "EXCL"



    # $ANTLR start "INC"
    def mINC(self, ):

        try:
            _type = INC
            _channel = DEFAULT_CHANNEL

            # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/cadlang.g:376:9: ( 'included' )
            # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/cadlang.g:376:11: 'included'
            pass
            self.match("included")



            self._state.type = _type
            self._state.channel = _channel

        finally:

            pass

    # $ANTLR end "INC"



    # $ANTLR start "LETTER"
    def mLETTER(self, ):

        try:
            # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/cadlang.g:378:19: ( 'a' .. 'z' | 'A' .. 'Z' | '_' )
            # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/cadlang.g:
            pass
            if (65 <= self.input.LA(1) <= 90) or self.input.LA(1) == 95 or (97 <= self.input.LA(1) <= 122):
                self.input.consume()
            else:
                mse = MismatchedSetException(None, self.input)
                self.recover(mse)
                raise mse





        finally:

            pass

    # $ANTLR end "LETTER"



    # $ANTLR start "DIGIT"
    def mDIGIT(self, ):

        try:
            # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/cadlang.g:380:19: ( '0' .. '9' )
            # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/cadlang.g:380:21: '0' .. '9'
            pass
            self.matchRange(48, 57)




        finally:

            pass

    # $ANTLR end "DIGIT"



    # $ANTLR start "IDENT"
    def mIDENT(self, ):

        try:
            _type = IDENT
            _channel = DEFAULT_CHANNEL

            # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/cadlang.g:382:9: ( ( LETTER | DIGIT )+ )
            # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/cadlang.g:382:11: ( LETTER | DIGIT )+
            pass
            # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/cadlang.g:382:11: ( LETTER | DIGIT )+
            cnt2 = 0
            while True: #loop2
                alt2 = 2
                LA2_0 = self.input.LA(1)

                if ((48 <= LA2_0 <= 57) or (65 <= LA2_0 <= 90) or LA2_0 == 95 or (97 <= LA2_0 <= 122)) :
                    alt2 = 1


                if alt2 == 1:
                    # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/cadlang.g:
                    pass
                    if (48 <= self.input.LA(1) <= 57) or (65 <= self.input.LA(1) <= 90) or self.input.LA(1) == 95 or (97 <= self.input.LA(1) <= 122):
                        self.input.consume()
                    else:
                        mse = MismatchedSetException(None, self.input)
                        self.recover(mse)
                        raise mse



                else:
                    if cnt2 >= 1:
                        break #loop2

                    eee = EarlyExitException(2, self.input)
                    raise eee

                cnt2 += 1





            self._state.type = _type
            self._state.channel = _channel

        finally:

            pass

    # $ANTLR end "IDENT"



    # $ANTLR start "TEXT"
    def mTEXT(self, ):

        try:
            _type = TEXT
            _channel = DEFAULT_CHANNEL

            # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/cadlang.g:384:9: ( '{' (~ '}' )* '}' )
            # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/cadlang.g:384:11: '{' (~ '}' )* '}'
            pass
            self.match(123)
            # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/cadlang.g:384:14: (~ '}' )*
            while True: #loop3
                alt3 = 2
                LA3_0 = self.input.LA(1)

                if ((0 <= LA3_0 <= 124) or (126 <= LA3_0 <= 65535)) :
                    alt3 = 1


                if alt3 == 1:
                    # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/cadlang.g:384:15: ~ '}'
                    pass
                    if (0 <= self.input.LA(1) <= 124) or (126 <= self.input.LA(1) <= 65535):
                        self.input.consume()
                    else:
                        mse = MismatchedSetException(None, self.input)
                        self.recover(mse)
                        raise mse



                else:
                    break #loop3


            self.match(125)



            self._state.type = _type
            self._state.channel = _channel

        finally:

            pass

    # $ANTLR end "TEXT"



    def mTokens(self):
        # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/cadlang.g:1:8: ( WS | COMMENT | AND | OR | NOT | T | F | LP | RP | LB | RB | SC | COM | TARROW | PERM | INPUT | MACRO | ENDMACRO | NAME | DEFAULT | WHEN | CONST | ENDCONST | SYNC | EXCL | INC | IDENT | TEXT )
        alt4 = 28
        alt4 = self.dfa4.predict(self.input)
        if alt4 == 1:
            # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/cadlang.g:1:10: WS
            pass
            self.mWS()


        elif alt4 == 2:
            # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/cadlang.g:1:13: COMMENT
            pass
            self.mCOMMENT()


        elif alt4 == 3:
            # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/cadlang.g:1:21: AND
            pass
            self.mAND()


        elif alt4 == 4:
            # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/cadlang.g:1:25: OR
            pass
            self.mOR()


        elif alt4 == 5:
            # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/cadlang.g:1:28: NOT
            pass
            self.mNOT()


        elif alt4 == 6:
            # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/cadlang.g:1:32: T
            pass
            self.mT()


        elif alt4 == 7:
            # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/cadlang.g:1:34: F
            pass
            self.mF()


        elif alt4 == 8:
            # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/cadlang.g:1:36: LP
            pass
            self.mLP()


        elif alt4 == 9:
            # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/cadlang.g:1:39: RP
            pass
            self.mRP()


        elif alt4 == 10:
            # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/cadlang.g:1:42: LB
            pass
            self.mLB()


        elif alt4 == 11:
            # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/cadlang.g:1:45: RB
            pass
            self.mRB()


        elif alt4 == 12:
            # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/cadlang.g:1:48: SC
            pass
            self.mSC()


        elif alt4 == 13:
            # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/cadlang.g:1:51: COM
            pass
            self.mCOM()


        elif alt4 == 14:
            # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/cadlang.g:1:55: TARROW
            pass
            self.mTARROW()


        elif alt4 == 15:
            # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/cadlang.g:1:62: PERM
            pass
            self.mPERM()


        elif alt4 == 16:
            # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/cadlang.g:1:67: INPUT
            pass
            self.mINPUT()


        elif alt4 == 17:
            # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/cadlang.g:1:73: MACRO
            pass
            self.mMACRO()


        elif alt4 == 18:
            # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/cadlang.g:1:79: ENDMACRO
            pass
            self.mENDMACRO()


        elif alt4 == 19:
            # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/cadlang.g:1:88: NAME
            pass
            self.mNAME()


        elif alt4 == 20:
            # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/cadlang.g:1:93: DEFAULT
            pass
            self.mDEFAULT()


        elif alt4 == 21:
            # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/cadlang.g:1:101: WHEN
            pass
            self.mWHEN()


        elif alt4 == 22:
            # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/cadlang.g:1:106: CONST
            pass
            self.mCONST()


        elif alt4 == 23:
            # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/cadlang.g:1:112: ENDCONST
            pass
            self.mENDCONST()


        elif alt4 == 24:
            # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/cadlang.g:1:121: SYNC
            pass
            self.mSYNC()


        elif alt4 == 25:
            # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/cadlang.g:1:126: EXCL
            pass
            self.mEXCL()


        elif alt4 == 26:
            # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/cadlang.g:1:131: INC
            pass
            self.mINC()


        elif alt4 == 27:
            # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/cadlang.g:1:135: IDENT
            pass
            self.mIDENT()


        elif alt4 == 28:
            # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/cadlang.g:1:141: TEXT
            pass
            self.mTEXT()







    # lookup tables for DFA #4

    DFA4_eot = DFA.unpack(
        u"\3\uffff\5\24\7\uffff\5\24\11\uffff\1\24\1\51\10\24\1\uffff\1\63"
        u"\1\uffff\1\64\7\24\3\uffff\1\76\2\24\1\101\3\24\3\uffff\1\105\1"
        u"\24\1\uffff\3\24\1\uffff\2\24\1\114\1\24\1\116\1\117\1\uffff\1"
        u"\24\2\uffff\1\121\1\uffff"
        )

    DFA4_eof = DFA.unpack(
        u"\122\uffff"
        )

    DFA4_min = DFA.unpack(
        u"\1\11\1\uffff\1\57\1\156\1\162\1\157\1\162\1\141\7\uffff\1\145"
        u"\1\150\1\171\1\170\1\156\6\uffff\1\156\2\uffff\1\144\1\60\1\164"
        u"\1\165\1\154\1\146\1\145\1\156\2\143\1\144\1\60\1\uffff\1\60\1"
        u"\145\1\163\1\141\1\156\1\143\2\154\1\143\2\uffff\1\60\1\145\1\165"
        u"\1\60\1\150\2\165\3\uffff\1\60\1\154\1\uffff\1\162\1\163\1\144"
        u"\1\uffff\1\164\1\157\1\60\1\145\2\60\1\uffff\1\144\2\uffff\1\60"
        u"\1\uffff"
        )

    DFA4_max = DFA.unpack(
        u"\1\173\1\uffff\1\160\1\156\1\162\1\157\1\162\1\141\7\uffff\1\145"
        u"\1\150\1\171\1\170\1\156\6\uffff\1\156\2\uffff\1\144\1\172\1\164"
        u"\1\165\1\154\1\146\1\145\1\156\2\143\1\144\1\172\1\uffff\1\172"
        u"\1\145\1\163\1\141\1\156\1\143\2\154\1\155\2\uffff\1\172\1\145"
        u"\1\165\1\172\1\150\2\165\3\uffff\1\172\1\154\1\uffff\1\162\1\163"
        u"\1\144\1\uffff\1\164\1\157\1\172\1\145\2\172\1\uffff\1\144\2\uffff"
        u"\1\172\1\uffff"
        )

    DFA4_accept = DFA.unpack(
        u"\1\uffff\1\1\6\uffff\1\10\1\11\1\12\1\13\1\14\1\15\1\16\5\uffff"
        u"\1\33\1\34\1\2\1\17\1\20\1\21\1\uffff\1\23\1\26\14\uffff\1\4\11"
        u"\uffff\1\3\1\5\7\uffff\1\22\1\27\1\6\2\uffff\1\25\3\uffff\1\7\6"
        u"\uffff\1\31\1\uffff\1\24\1\30\1\uffff\1\32"
        )

    DFA4_special = DFA.unpack(
        u"\122\uffff"
        )


    DFA4_transition = [
        DFA.unpack(u"\2\1\25\uffff\1\1\7\uffff\1\10\1\11\2\uffff\1\15\1\16"
        u"\1\uffff\1\2\12\24\1\uffff\1\14\5\uffff\32\24\1\12\1\uffff\1\13"
        u"\1\uffff\1\24\1\uffff\1\3\2\24\1\17\1\22\1\7\2\24\1\23\4\24\1\5"
        u"\1\4\3\24\1\21\1\6\2\24\1\20\3\24\1\25"),
        DFA.unpack(u""),
        DFA.unpack(u"\1\26\63\uffff\1\34\1\uffff\1\32\3\uffff\1\30\3\uffff"
        u"\1\31\1\33\1\uffff\1\27"),
        DFA.unpack(u"\1\35"),
        DFA.unpack(u"\1\36"),
        DFA.unpack(u"\1\37"),
        DFA.unpack(u"\1\40"),
        DFA.unpack(u"\1\41"),
        DFA.unpack(u""),
        DFA.unpack(u""),
        DFA.unpack(u""),
        DFA.unpack(u""),
        DFA.unpack(u""),
        DFA.unpack(u""),
        DFA.unpack(u""),
        DFA.unpack(u"\1\42"),
        DFA.unpack(u"\1\43"),
        DFA.unpack(u"\1\44"),
        DFA.unpack(u"\1\45"),
        DFA.unpack(u"\1\46"),
        DFA.unpack(u""),
        DFA.unpack(u""),
        DFA.unpack(u""),
        DFA.unpack(u""),
        DFA.unpack(u""),
        DFA.unpack(u""),
        DFA.unpack(u"\1\47"),
        DFA.unpack(u""),
        DFA.unpack(u""),
        DFA.unpack(u"\1\50"),
        DFA.unpack(u"\12\24\7\uffff\32\24\4\uffff\1\24\1\uffff\32\24"),
        DFA.unpack(u"\1\52"),
        DFA.unpack(u"\1\53"),
        DFA.unpack(u"\1\54"),
        DFA.unpack(u"\1\55"),
        DFA.unpack(u"\1\56"),
        DFA.unpack(u"\1\57"),
        DFA.unpack(u"\1\60"),
        DFA.unpack(u"\1\61"),
        DFA.unpack(u"\1\62"),
        DFA.unpack(u"\12\24\7\uffff\32\24\4\uffff\1\24\1\uffff\32\24"),
        DFA.unpack(u""),
        DFA.unpack(u"\12\24\7\uffff\32\24\4\uffff\1\24\1\uffff\32\24"),
        DFA.unpack(u"\1\65"),
        DFA.unpack(u"\1\66"),
        DFA.unpack(u"\1\67"),
        DFA.unpack(u"\1\70"),
        DFA.unpack(u"\1\71"),
        DFA.unpack(u"\1\72"),
        DFA.unpack(u"\1\73"),
        DFA.unpack(u"\1\75\11\uffff\1\74"),
        DFA.unpack(u""),
        DFA.unpack(u""),
        DFA.unpack(u"\12\24\7\uffff\32\24\4\uffff\1\24\1\uffff\32\24"),
        DFA.unpack(u"\1\77"),
        DFA.unpack(u"\1\100"),
        DFA.unpack(u"\12\24\7\uffff\32\24\4\uffff\1\24\1\uffff\32\24"),
        DFA.unpack(u"\1\102"),
        DFA.unpack(u"\1\103"),
        DFA.unpack(u"\1\104"),
        DFA.unpack(u""),
        DFA.unpack(u""),
        DFA.unpack(u""),
        DFA.unpack(u"\12\24\7\uffff\32\24\4\uffff\1\24\1\uffff\32\24"),
        DFA.unpack(u"\1\106"),
        DFA.unpack(u""),
        DFA.unpack(u"\1\107"),
        DFA.unpack(u"\1\110"),
        DFA.unpack(u"\1\111"),
        DFA.unpack(u""),
        DFA.unpack(u"\1\112"),
        DFA.unpack(u"\1\113"),
        DFA.unpack(u"\12\24\7\uffff\32\24\4\uffff\1\24\1\uffff\32\24"),
        DFA.unpack(u"\1\115"),
        DFA.unpack(u"\12\24\7\uffff\32\24\4\uffff\1\24\1\uffff\32\24"),
        DFA.unpack(u"\12\24\7\uffff\32\24\4\uffff\1\24\1\uffff\32\24"),
        DFA.unpack(u""),
        DFA.unpack(u"\1\120"),
        DFA.unpack(u""),
        DFA.unpack(u""),
        DFA.unpack(u"\12\24\7\uffff\32\24\4\uffff\1\24\1\uffff\32\24"),
        DFA.unpack(u"")
    ]

    # class definition for DFA #4

    DFA4 = DFA




def main(argv, stdin=sys.stdin, stdout=sys.stdout, stderr=sys.stderr):
    from antlr3.main import LexerMain
    main = LexerMain(cadlangLexer)
    main.stdin = stdin
    main.stdout = stdout
    main.stderr = stderr
    main.execute(argv)


if __name__ == '__main__':
    main(sys.argv)



## Filename    : MCLSolutions.py
## Author(s)   : Michel Le Borgne
## Created     : 03/9/2012
## Revision    :
## Source      :
##
## Copyright 2012 : IRISA/IRSET
##
## This library is free software; you can redistribute it and/or modify it
## under the terms of the GNU General Public License as published
## by the Free Software Foundation; either version 2.1 of the License, or
## any later version.
##
## This library is distributed in the hope that it will be useful, but
## WITHOUT ANY WARRANTY, WITHOUT EVEN THE IMPLIED WARRANTY OF
## MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.  The software and
## documentation provided here under is on an "as is" basis, and IRISA has
## no obligations to provide maintenance, support, updates, enhancements
## or modifications.
## In no event shall IRISA be liable to any party for direct, indirect,
## special, incidental or consequential damages, including lost profits,
## arising out of the use of this software and its documentation, even if
## IRISA have been advised of the possibility of such damage.  See
## the GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this library; if not, write to the Free Software Foundation,
## Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.
##
## The original code contained here was initially developed by:
##
##     Michel Le Borgne.
##     IRISA
##     Symbiose team
##     IRISA  Campus de Beaulieu
##     35042 RENNES Cedex, FRANCE
##
##
## Contributor(s): Geoffroy Andrieux - IRISA/IRSET
##
"""
Various solution representations
"""

# C++ API
from _cadbiom import get_unshift_code, unflatten

class MCLException(Exception):
    """
    Exception for MCLAnalyser
    """
    def __init__(self, mess):
        self.message = mess

    def __str__(self):
        return self.message

class RawSolution(object):
    """
    RawSolution objects contain a solution got from SAT solver with all
    variable parameters from the unfolder. This permit different extractions.
    IMPORTANT: the solution MUST correspond to the properties registered
    for its computation. Any change in these properties will give incorrect
    results.
    """
    def __init__(self, sol, cl_unfolder):
        """
        @param sol: DIMACS encoded solution
        @param cl_unfolder: the cl_unfolder (to be used for invariant constants)
        """
        self.__solution = sol
        self.__unfolder = cl_unfolder
        self.__nb_steps = cl_unfolder.get_current_step()
        self.__shift_step = cl_unfolder.get_shift_step() # dependant on the run
        self.__current_step = cl_unfolder.get_current_step()
        self.__shift_direction = cl_unfolder.get_shift_direction()

    def get_unshift_code(self, var_num):
        """
        @param var_num: DIMACS literal coding of a shifted variable x_i
        @return: DIMACS literal coding of  x_0 with same value
        """

        # Old API
        # var_code = (abs(var_num) - 1) % self.__shift_step + 1
        #
        # if var_code <= self.__shift_step:
        #     return var_code * (-1 if var_num < 0 else 1)
        # else:
        #     raise MCLException("Not a DIMACS code of an initial variable")

        # New API via C++ module
        get_unshift_code(var_num, self.__shift_step)

    def get_var_name(self, var_num):
        """
        For translation to symbolic representation
        """
        return self.__unfolder.get_var_name(var_num)

    def get_solution_size(self):
        """
        @return: int
        """
        return len(self.__solution)

    def get_solution(self):
        """
        DIMACS encoding of the solution from the solver
        """
        return self.__solution

    def get_nbsteps(self):
        """
        @return: int
        """
        return self.__nb_steps

    def get_unfolder(self):
        """
        @return: a clunfolder
        """
        return self.__unfolder

    def unflatten(self):
        """
        Transform a flat DIMACS representation of a trajectory into a list<list<int>>
        each sublist represents a state vector. Auxiliary variables introduced by properties
        compiling are ignored.
        @return: a list of state vectors (list<int>) in DIMACS format
        """

        # Old API
        # lv_traj = []
        # step = 0
        # dec = 0
        # len_shift_step_init = self.__unfolder.get_shift_step_init()
        # # n steps means h0, ..., h(n-1) in constraints for clocks and inputs
        # while dec + self.__shift_step <  len(self.__solution):
        #     #assert [self.get_unshift_code(self.__solution[i+dec]) for i in range(len_shift_step_init)] == [get_unshift_code(self.__solution[i+dec], self.__shift_step) for i in range(len_shift_step_init)]
        #     lv_traj.append(
        #         [get_unshift_code(self.__solution[i+dec], self.__shift_step)
        #          for i in range(len_shift_step_init)]
        #     )
        #     step += 1
        #     dec = step * self.__shift_step # first index of time step
        # return lv_traj # list of lists

        # New API via C++ module
        return unflatten(
            self.__solution,
            self.__unfolder.get_shift_step_init(),
            self.__shift_step
        )


    def extract_frontier_values(self):
        """
        Extract values of the init frontier variables from a whole solution vector
        Assert: sol is sorted by variable code

        @return: a dimacs code list of the activation state on the frontier for sol
        """

        # Old API
        # jmax = len(self.__unfolder.get_frontier())
        # fsol = []
        # j = 0
        # dec = self.__shift_step * self.__current_step
        # sol = self.__solution
        # for i in range(len(sol)):
        #     # the variable is frontier var:
        #     if abs(sol[i]) == self.__unfolder.get_frontier()[j]:
        #         if self.__shift_direction == "FORWARD":
        #             fsol.append(sol[i])
        #         else:
        #             fsol.append(sol[i+dec])     # look for initial values
        #         j += 1
        #         if j == jmax:
        #             return fsol
        # return fsol # never reach this point

        frontiers = self.__unfolder.get_frontier()
        jmax = len(frontiers)
        fsol = list()
        j = 0
        dec = self.__shift_step * self.__current_step
        for i, sol in enumerate(self.__solution):
            # the variable is frontier var:
            if abs(sol) == frontiers[j]:
                if self.__shift_direction == "FORWARD":
                    fsol.append(sol)
                else:
                    fsol.append(self.__solution[i+dec])# look for initial values
                j += 1
                if j == jmax:
                    return fsol
        return fsol # never reach this point

    def extract_act_frontier_values(self, sol):
        """
        frontier variables which are active at the beginning of  whole solution vector
        Assert: sol is sorted by variable code
        @param sol:  a dimacs solution
        @return: a dimacs code list of the activation state on the frontier for sol
        """
        frontiers = self.__unfolder.get_frontier()
        jmax = len(frontiers)
        fsol = list()
        j = 0
        dec = self.__shift_step * self.__current_step
        for i, sol in enumerate(self.__solution):
            # the variable is frontier var:
            if abs(sol) == frontiers[j]:
                if self.__shift_direction == "FORWARD":
                    if sol > 0:
                        fsol.append(sol)
                else:
                    if self.__solution[i+dec] > 0:
                        fsol.append(self.__solution[i+dec])# look for initial values
                j += 1
                if j == jmax:
                    return fsol
        return fsol # never reach this point


    def extract_act_inputs_clocks(self, s_vector):
        """
        extract active __inputs and clocks from a state vector
        """

        inputs      = frozenset(self.__unfolder.get_inputs())
        free_clocks = frozenset(self.__unfolder.get_free_clocks())

        return [s_varcode for s_varcode in s_vector
                if (s_varcode > 0)
                and ((s_varcode in inputs)
                    or (s_varcode in free_clocks))]


    def extract_act_input_clock_seq(self):
        """
        Extract the sequence of activated __inputs and clocks in the solution sol
        @return: list<list<int>> list of input vector
        """

        return [] \
            if (len(self.__unfolder.get_inputs()) == 0) and (len(self.__unfolder.get_free_clocks()) == 0) \
            else [self.extract_act_inputs_clocks(xic)
                  for xic in self.unflatten()] # list of lists

    def __str__(self):
        """
        For debug purpose
        """
        unfl = self.unflatten()
        stn = []
        for vect in unfl:
            vect_n = []
            for dcod in vect:
                if dcod > 0:
                    vect_n.append(self.get_var_name(dcod))
                else:
                    vect_n.append("-"+self.get_var_name(dcod))
            stn.append(vect_n)
        return stn.__str__()

    def display_activ_sol(self):
        """
        for debug
        """
        unfl = self.unflatten()
        stn = []
        for vect in unfl:
            vect_n = []
            for dcod in vect:
                if dcod > 0:
                    v_name = self.get_var_name(dcod)
                    if not v_name[0] == "_":
                        vect_n.append(v_name)
            vect_n.sort()
            stn.append(vect_n)
        print stn



class FrontierSolution(object):
    """
    class for symbolic frontier and timing representation (built from a raw solution)
    """

    def __init__(self, act_frontier, ic_seq, horizon):
        """
        Build a symbolic representation of frontier values and timing from
        a raw solution
        """
        # attributes
        self.activated_frontier = act_frontier   # list<string>
        self.ic_sequence = ic_seq                # list<string> (format % h1 h2)
        self.horizon = horizon                   # int



    @classmethod
    def from_raw(cls, raws):
        """
        Build a symbolic representation of frontier values and timing from
        a raw solution
        @param raws: a raw solution
        """
        # activated frontier places
        dim_front = raws.extract_frontier_values()
        out = []
        for dcod in dim_front:
            if dcod > 0:
                out.append(raws.get_var_name(dcod))
        activated_frontier = out

        # activated input/clocks
        ic_seq = raws.extract_act_input_clock_seq()
        lic = []
        for icv in ic_seq:
            ic_act = "%"
            for iii in icv:
                ic_act = ic_act + " " + raws.get_var_name(iii)
            lic.append(ic_act)
        ic_sequence = lic
        horizon = raws.get_nbsteps()
        return cls(activated_frontier, ic_sequence, horizon)

    @classmethod
    def from_dimacs_front_sol(cls, fds):
        """
        Build a symbolic representation of frontier values and timing from
        a DimacsFrontierSolution solution
        @param fds: a raw solution
        """
        # activated frontier places
        dim_front = fds.frontier_values
        out = []
        for dcod in dim_front:
            if dcod > 0:
                out.append(fds.get_var_name(dcod))
        activated_frontier = out

        # activated input/clocks
        ic_seq = fds.ic_sequence
        lic = []
        for icv in ic_seq:
            ic_act = "%"
            for iii in icv:
                ic_act = ic_act + " " + fds.get_var_name(iii)
            lic.append(ic_act)
        ic_sequence = lic
        horizon = fds.get_nbsteps()
        return cls(activated_frontier, ic_sequence, horizon)

    @classmethod
    def from_file(cls, file_name):
        """
        Build a symbolic representation of frontier values and timing from
        a DimacsFrontierSolution solution
        """
        try:
            fr_f = open(file_name)
        except:
            raise MCLException("Impossible to open file: "+file_name)
        act_front = fr_f.readline().split()
        ic_seq = []
        step = fr_f.readline()
        horizon = 0
        while step:
            horizon += 1
            ic_seq.append(step[:-1])
            step = fr_f.readline()
        return cls(act_front, ic_seq, horizon)

    def __str__(self):
        """
        give a string representation of a Frontier solution
        """
        stro = "\nActivated front.:"
        stro = stro + self.activated_frontier.__str__()
        stro = stro + "\nTiming:"
        for icc in self.ic_sequence:
            stro =  stro + "\n   " + icc
        return stro

    def has_same_frontier(self, other):
        """
        Check if two FrontierSol have same frontier
        @param other: a DimacsFrontierSol
        @return bool
        """
        return set(self.frontier_values) == set(other.frontier_values)

    def save(self, outfile):
        """
        Save a symbolic solution in a file
        The format is readable by the simulator
        @param outfile: writable file
        """
        outfile.write(' '.join(sorted(self.activated_frontier, key=lambda s: s.lower())) + '\n')
        outfile.write('\n'.join(self.ic_sequence) + '\n')
        outfile.flush()


class DimacsFrontierSol(object):
    """
    class for solution frontier and timing representation
    (built from a raw solution) in Dimacs code
    """
    def __init__(self, raws):
        """
        Build a numeric representation of frontier values and timing from
        a raw solution
        @param raws: a raw solution
        """
        # attributes
        # self.frontier_values : list<int>
        # self.ic_sequence     : list<list<int>>
        # self.horizon         : int

        # frontier places
        self.frontier_values = raws.extract_frontier_values()

        # activated input/clocks (may be empty if data-flow model without input)
        self.ic_sequence = raws.extract_act_input_clock_seq()
        self.horizon = raws.get_nbsteps()

        # CLUnfolder
        self.__unfolder = raws.get_unfolder()

    def get_var_name(self, var_num):
        """
        For translation to symbolic representation
        """
        return self.__unfolder.get_var_name(var_num)

    def get_solution_size(self):
        """
        @return: int
        """
        return len(self.frontier_values)

    def get_solution(self):
        """
        @return: list<int>
        """
        return self.frontier_values

    def get_nbsteps(self):
        """
        @return: int
        """
        return self.horizon

    def has_same_frontier(self, other):
        """
        Check if two DimacsFrontierSol have same frontier
        @param other: a DimacsFrontierSol
        @return bool
        """
        return set(self.frontier_values) == set(other.frontier_values)

    def front_is_in(self, ldfs):
        """
        Check if an object of ldfs has the same frontier list
        @param ldfs: list of  DimacsFrontierSol
        @return bool
        """
        for dfs in ldfs:
            #if self.has_same_frontier(dfs):
            if set(self.frontier_values) == set(dfs.frontier_values):
                return True
        return False

    def nb_activated(self):
        """
        Count the number of activated places in solution
        @return: int
        """
        cpt = 0
        for vcod in self.frontier_values:
            if vcod > 0:
                cpt += 1
        return cpt

    def nb_timed(self):
        """
        Count the number of events in solution
        @return: int
        """
        cpt = 0
        for ic_act in self.ic_sequence:
            cpt += len(ic_act)
        return cpt

    def display(self):
        """
        For tests and debug
        """
        print 'Frontier solution:'
        print self.frontier_values

# useful functions for solution lists
def extract_dimacs_init_frontier(lsol):
    """
    Extract init values of the frontier variables from a whole solution vector for
    a list of solution vectors.
    Eliminate double frontier level
    @param lsol: list of raw solutions
    @return: a list of DimacsFrontierSol
    """
    ext1 = []
    for sol in lsol:
        esol = DimacsFrontierSol(sol)
        if not esol.front_is_in(ext1):
            ext1.append(esol)
    return ext1

